 /*
  * hashicmp.h
  *
  * ip srcs that generate icmp errors
  */

#define MAXUDPPORTS	10

#include "hwflags.h"

/* a 5-tuple, ip src, total icmp errors, unreachables, redirects, ttl
*/
struct icmpnode {
	unsigned int ipsrc; 		/* index */
	unsigned int tcpcount;		/* total # of tcp packets */
	unsigned int udpcount;		/* total # of udp packets */
	unsigned int udprcv;		/* total # of udp packets recv */
	unsigned int icmpcount;	/* total # of icmp packets */
	unsigned int totalicmp;	/* icmp: sort on this */
	unsigned int udpww;		/* udp: sort on this */
	unsigned int unreach;		/* mention this */
	unsigned int redirect;		/* mention this */
	unsigned int ttl;		/* mention this */
	unsigned int l3count;		/* count of unique l3 destinations */
	unsigned int l4count;		/* count of unique l4 destinations */
	unsigned int ipdst;		/* sampled ip dst for udp portreport */
	unsigned int udpflags;         /* P flag and room for growth */
	unsigned int udp_bytes;	/* total UDP byte count SENT */
	unsigned int sizes[6];         /* pkt size distribution */
/* experimental pkt size distribution and payload avg attributes
*/
	unsigned int udp_rbytes;        /* to host - count of pay. pkts size */
/* end new attributes
*/
	struct tags apptags; 		/* managed by patmatch*/
	int pcount;			/* number of dst ports counted */
	int spcount;			/* number of src ports counted */
	int dports[MAXUDPPORTS];   	/* max dst ports */
	int dpcounts[MAXUDPPORTS];   	/* per port hit count */
	int sports[MAXUDPPORTS];   	/* max src ports */
	unsigned int spcounts[MAXUDPPORTS];   /* associated src port counters */
	struct icmpnode *next;
};

#define ICMPHSIZE 104729
struct icmplist{
	unsigned int entries;
	struct icmpnode *last;
	struct icmpnode *hashlist[ICMPHSIZE];
};

/**************************************************/
/* function prototypes */

void init_icmplist(struct icmplist *l);

struct icmpnode *search_srcicmp(struct t_entry *te, struct icmplist *l, unsigned int ipsrc, 
	int tcpcount, int udpcount, int icmpcount,int,int,int,int, unsigned int udpport, int honeyflag);

int insert_srcicmp(struct t_entry *te, struct icmplist *l, unsigned int ipsrc, int tcpcount, int udpcount, int icmpcount, unsigned int udpport, int honeyflag);
void destroy_icmplist(struct icmplist *l);
void print_icmplist(FILE *fd, struct icmplist *l, int count, int scanflag, struct slist *, struct slist *);


