 /*
  * hashsyn.h
  *
  * go forth and syn no more.
  */

/*
 * notes on this data structure
 *	fins: a large amount of syns and a small amount of fins is a NEGATIVE indicator
 *		we either have an attacker or an ineffective protocol.
 *		equal amounts of syns/fins may be regarded as a good sign.
 *		NEGATIVE if small, and/or syns significantly greater.
 *
 *	rst: non-trivial rsts, are of course a bad sign. 
 *		NEGATIVE.
 *
 *	tcpfrom/tcpto are a measure of work.  small tcpto is a bad sign.
 *		work. NEGATIVE if from greater than to.
 *
 *	icmpcount: icmp errors that we see in the real world.  also a negative indicator.
 *		NEGATIVE.
 */
#define WEIGHTCOUNTER	float

/* info flag bits for this ip src address */
#define P2PBITS		0x2ff		/* do not include port flags */
#define IRCBITS		0xf		/* irc traffic indicated */

/* old hardwired tags here
*/
#include "hwflags.h"

#define MAXPORTS			10
struct synnode {
	unsigned int ipsrc; 		/* index */
	unsigned int dstip;		/* 1 sampled ip dst */
	unsigned int syncount;		/* syns sent by ip src */
	unsigned int fincount;		/* fins returned to ip src */
	unsigned int finssent;		/* fins sent by ip src */
	unsigned int rstcount;		/* resets returned to ip src */
	unsigned int sacount;		/* number of syn+ack pairs sent by ip src */
	unsigned int tcpfrom;		/* total tcp packets sent from ipsrc */
	unsigned int tcpto;		/* total tcp pkts sent to ip src */
	unsigned int icmpcount;	/* total icmp errors sent back to the ip src */ 
	unsigned int flags;		/* flags related to ip src */
	unsigned int emulehits;	/* count of emule guesses */
	unsigned int ipds;		/* ip dst count */
	unsigned int tpds;		/* tcp port count */
	unsigned int bytecount; 	/*total number of bytes sent and received by host*/
	unsigned int sizes[4];		/* primitive size distribution */
	/* the following 4 items do not appear in the output tuple
	*/
	unsigned int p25_syn;		/* syns sent to port 25 */
	unsigned int p25_fin;		/* fins returned to port 25 */
	unsigned int p25_rst;		/* rsts returned to port 25 */
	unsigned int p25_total;	/* total port 25 pkts */
	/* p25ww does appear in output tuple
	*/
	int p25ww;			/* ww for port 25 by itself */
	WEIGHTCOUNTER work;		/* work weight + a little fudge*/
	WEIGHTCOUNTER ww;		/* work weight with no fudge at all */
	int pcount;			/* number of ports counted */
	int dports[MAXPORTS];   	/* max dst ports */
	unsigned int dpcounts[MAXPORTS];   /* associated dst port counters */
	struct tags apptags; /*struct for application flags - managed by patmatch*/
	int scount;			/* number of ports counted */
	int sports[MAXPORTS];   	/* max src ports */
	unsigned int spcounts[MAXPORTS];   /* associated src port counters */
	struct synnode *next;
};

#define THSIZE 104729

struct synlist{
	unsigned int entries;
	struct synnode *last;
	struct synnode *hashlist[THSIZE];
};

/*
static unsigned int e_hash(unsigned char *, int size);
*/

/**************************************************/
/* function prototypes */

void init_synlist(struct synlist *l);

void search_srcsyn(struct t_entry *te, struct synlist *l, int saflag, int ircresult, int icmpflag);

void destroy_synlist(struct synlist *l);
void print_synlist(FILE *fd, struct synlist *l, int count, int wflag, 
	char *wormfile, int ipaflag, int scanflag, struct slist *isl, struct slist *tsl);
