/*
 *	Copyright (c) 1993-1997 JSC Rinet, Novosibirsk, Russia
 *
 * Redistribution and use in source forms, with and without modification,
 * are permitted provided that this entire comment appears intact.
 * Redistribution in binary form may occur without any restrictions.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT ANY WARRANTIES OF ANY KIND.
 */

/*
 * Copyright (c) 2001 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* interfaces.h -- network interface data-link level routines */

#ifdef	HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/types.h>
#ifdef	HAVE_SYS_MBUF_H
#include <sys/mbuf.h>
#endif
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#ifdef	HAVE_NET_SLIP_H
#include <net/slip.h>
#include <net/slcompress.h>
#endif
#include <syslog.h>
#include <pcap.h>
#include <string.h>

#include "ourmon.h"
#include "trigger.h"

#include <stdio.h>
#include <stdlib.h>

u_char *snapend;

/* Link-level header length */
#ifndef	ETHER_HDRLEN
#define	ETHER_HDRLEN	sizeof(struct ether_header)
#endif

#ifndef	SLIP_HDRLEN
#ifdef	SLC_BPFHDR
#define	SLIP_HDRLEN	SLC_BPFHDR	/* if _BSDI_VERSION >= 199510 */
#else
#define	SLIP_HDRLEN	16	/* bpf slip header length */
#endif
#endif

#ifndef	PPP_HDRLEN
#ifdef	SLC_BPFHDR
#define	PPP_HDRLEN	SLC_BPFHDR	/* if _BSDI_VERSION >= 199510 */
#else
#define	PPP_HDRLEN	4	/* sizeof(struct ppp_header) */
#endif
#endif

#ifdef	DLT_C_HDLC
#ifndef	CHDLC_HDRLEN
#define	CHDLC_HDRLEN	4	/* sizeof(struct cisco_hdr) */
#endif
#endif

#ifndef	NULL_HDRLEN
#define	NULL_HDRLEN	4	/* loopback header length */
#endif

extern int any_bpf;
extern unsigned int trigger_worm_state;	

/*
 * interface dependent routings, basically called thru jump table
 * (DLT type).  if_ether is for ethernet.
 * user - is string passed from main level thru pcap loop routine.
 * h - pcap header structure
 * p - the packet itself
 *	Sub-variables:
 *	length - length of the packet
 *	caplen - bytes actually captured.
 */
void
if_ether(user, h, p)
	u_char *user;
	struct pcap_pkthdr *h;
	u_char *p;
{
	extern int debug;
	struct t_entry tentry;		/* packet info summarization structure */
	struct t_entry *te = &tentry;

	u_int caplen = h->caplen;	/* amt. actually captured */
	u_int length = h->len;		/* length of packet */

	te->sizeindex = te->ipflag = 0;
#ifdef EDEBUG
	if (debug) {
		printf("if_ether: caplen %d, length %d, p %x\n", caplen, length, p);
	}
#endif

	if (caplen < ETHER_HDRLEN) 
		return;

	/* snapend is pointer to end of packet buffer
	*/
	snapend = p + caplen;

	/* if 802.1Q we must adjust the packet.  we move the ethernet hdr up
	 */
	if ( ntohs(((struct ether_header *)p)->ether_type) == 0x8100) {
		char buf[12];
		
		memcpy(buf, p, 12);
		p += 4;
		memcpy(p, buf, 12);	/* overwrite the 4 bytes of vlan tag */
		caplen -= 4;
		length -= 4;
		h->caplen -= 4;
		h->len -= 4;
		if (debug) {
			printf("xyz802.1Q packet, type is %x\n",  
				ntohs(((struct ether_header *)p)->ether_type)); 
		}
	}
	else {
#ifdef EDEBUG
		if (debug) {
			printf("xyz not 802.1Q packet, type is %x\n", 
				ntohs(((struct ether_header *)p)->ether_type)); 
		}
#endif
	}

	/* for ethernet we have a special function to deal with L2 only stats,
	 * else this is somewhat kludgily dealt with by the analyze_ip function
	 * with the case of other DLT types (not ethernet though).
	 */
	analyze_nonip((char *)(p + ETHER_HDRLEN), length - ETHER_HDRLEN, length,
			(struct ether_header *)p, te);

	/* run top-n and other IP only filters
	*/
	if ( ntohs(((struct ether_header *)p)->ether_type) == ETHERTYPE_IP) {
		analyze_ip(h, p, (struct ip *) (p + ETHER_HDRLEN), length - ETHER_HDRLEN, length,
			(struct ether_header *)p, te);
	}
	
#ifdef EDEBUG
	else {
		if (debug) {
			printf("xyz not IP packet, type is %x\n", ntohs(((struct ether_header *)p)->ether_type)); 
		}
	}
#endif

	/* do bpf interpreter (and cbpf)
	 * note that te may have been filled in by analyze_ip in this case.
	 * CBPFs are only executed if analyze_ip decides that the packet
	 * is a sane ip packet.
	*/
	if (any_bpf) {
		analyze_bpf(h, p, caplen, length, te);
	}
	
	/* execute triggers if any
	 * TBD. needs lock
	*/
	if (trigger_worm_state != 0)  {	
		trigger_work(h, p, caplen, length); 
	}
}

void
if_slip(user, h, p)
	u_char *user;
	struct pcap_pkthdr *h;
	register u_char *p;
{
	struct t_entry tentry;
	struct t_entry *te = &tentry;

	u_int caplen = h->caplen;
	u_int length = h->len;
	te->sizeindex = te->ipflag = 0;

	if (caplen < SLIP_HDRLEN) 
		return;

	snapend = p + caplen;
	analyze_ip(h, p, (struct ip *)(p + SLIP_HDRLEN)
		, length - SLIP_HDRLEN, length, (struct ether_header *) 
		NULL, te);

	if (any_bpf) {
		analyze_bpf(h, p, caplen, length, te);
	}

	if (trigger_worm_state != 0)  {	
		trigger_work(h, p, caplen, length); 
	}
}

void
if_ppp(user, h, p)
	u_char *user;
	struct pcap_pkthdr *h;
	register u_char *p;
{
	u_int caplen = h->caplen;
	u_int length = h->len;
	register hdrlen = 0;
	u_short type;
	u_char *packetp;
	struct t_entry tentry;
	struct t_entry *te = &tentry;

	if (caplen < PPP_HDRLEN) return;

	packetp = p;
	snapend = p + caplen;
	te->sizeindex = te->ipflag = 0;

#ifdef	SLC_BPFHDRLEN
	p += SLC_BPFHDRLEN;	/* pointer to link level header */
#endif
	/* PPP address and PPP control fields may be present (-acfc) */
	if (p[0] == 0xff && p[1] == 0x03) {
		p += 2;
		hdrlen += 2;
	}
	/* Retrieve the protocol type */
	if (*p & 01) {	/* Compressed protocol field (pfc) */
		type = *p++;
		hdrlen++;
	} else {	/* Un-compressed protocol field (-pfc) */
		type = ntohs(*(u_short *)p);
		p += 2;
		hdrlen += 2;
	}
	if (type == 0x21) {	/* IP protocol */
#ifdef	SLC_BPFHDR
		p = packetp + SLC_BPFHDR;	/* skip bpf pseudo header */
		hdrlen = SLC_BPFHDR;
#endif
		analyze_ip(h, p, (struct ip *)p, length - hdrlen, length, 
			(struct ether_header *) NULL, te);
		if (any_bpf) {
			analyze_bpf(h, p, caplen, length, te);
		}
	}

	if (trigger_worm_state != 0)  {	
		trigger_work(h, p, caplen, length); 
	}
}

#ifdef	DLT_C_HDLC
void
if_chdlc(user, h, p)
	char *user;
	struct pcap_pkthdr *h;
	register u_char *p;
{
	u_int caplen = h->caplen;
	u_int length = h->len;
	struct t_entry tentry;
	struct t_entry *te = &tentry;

	if (caplen < CHDLC_HDRLEN) return;
	te->sizeindex = te->ipflag = 0;

	snapend = p + caplen;
	if (ntohs(*(u_short *)(p + 2)) == 0x0800) /* IP protocol */
		analyze_ip(h, p, (struct ip *)p + CHDLC_HDRLEN, length - CHDLC_HDRLEN, length,
			NULL, te);

	if (any_bpf) {
		analyze_bpf(h, p, caplen, length, te);
	}

	if (trigger_worm_state != 0)  {	
		trigger_work(h, p, caplen, length); 
	}
}
#endif

#ifdef	DLT_RAW
void
if_rawip(user, h, p)
	u_char *user;
	struct pcap_pkthdr *h;
	register u_char *p;
{
	u_int caplen = h->caplen;
	u_int length = h->len;
	struct t_entry tentry;
	struct t_entry *te = &tentry;
	te->sizeindex = te->ipflag = 0;

	snapend = p + caplen;
	analyze_ip(h, p, (struct ip *)p, length, length, NULL, te);
	if (any_bpf) {
		analyze_bpf(h, p, caplen, length, te);
	}

	if (trigger_worm_state != 0)  {	
		trigger_work(h, p, caplen, length); 
	}
}
#endif

void
if_null(user, h, p)
	u_char *user;
	struct pcap_pkthdr *h;
	register u_char *p;
{
	u_int caplen = h->caplen;
	u_int length = h->len;
  	u_int family;
	struct t_entry tentry;
	struct t_entry *te = &tentry;
	te->sizeindex = te->ipflag = 0;
	
	if (debug) {
		printf("lo0: caplen %d, length %d\n", caplen, length);
	}

	memcpy(&family, p, sizeof(family));
	snapend = p + caplen;
	if (family == AF_INET)
		analyze_ip(h, p, (struct ip *)(p + NULL_HDRLEN), length - NULL_HDRLEN, 
				length, NULL, te);

	if (any_bpf) {
		analyze_bpf(h, p, caplen, length, te);
	}

	if (trigger_worm_state != 0)  {	
		trigger_work(h, p, caplen, length); 
	}
}

/*
 * hardware dependent functions table.
 */
static struct if_func {
	void (*f)();
	int type;
} if_funcs[] = {
	{	if_ether,	DLT_EN10MB	},	/* Ethernet */
#ifdef	DLT_IEEE802
	{	if_ether,	DLT_IEEE802	},	/* IEEE 802 */
#endif
	{	if_slip,	DLT_SLIP	},	/* SLIP */
#ifdef	DLT_SLIP_BSDOS
	{	if_slip,	DLT_SLIP_BSDOS	}, /* libpcap stupid fake */
#endif
	{	if_ppp,		DLT_PPP		},	/* PPP */
#ifdef	DLT_PPP_BSDOS
	{	if_ppp,		DLT_PPP_BSDOS	}, /* libpcap stupid fake */
#endif
#ifdef	DLT_C_HDLC
	{	if_chdlc,	DLT_C_HDLC	},	/* Cisco HDLC */
#endif
#ifdef	DLT_RAW
	{	if_rawip,	DLT_RAW		},	/* raw IP */
#endif
	{	if_null,	DLT_NULL	},	/* loopback */
	{ NULL, 0 },
};

/*
 * Assign data link type to interface function.
 */
pcap_handler
lookup_if(type)
	int type;
{
	struct if_func *p;

	for (p = if_funcs; p->f != NULL; ++p)
		if (type == p->type) return p->f;
	error(0, "unknown data link type 0x%x", type);

	/* NOTREACHED */
	return 0;
}
