#ifdef THREAD
/*
 * Copyright (c) 1993, 1994, 1995, 1996, 1997, 1998
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Copyright (c) 2007 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/socket.h>

#ifdef BSD
#include <net/bpf.h>
#include <sys/filio.h>
#else
#include <asm/ioctls.h>
#endif
#include <pcap.h>
#include <pcap-int.h>

#include "ourpcap.h"
#include "sync.h"

extern void doreport();
extern void signal_block();
extern void signal_unblock();

extern int debug;

int ourpcap_loop(pcap_t *p, int cnt, pcap_handler callback, u_char *user)
{
	register int n;

#ifdef BSD
	p->read_op = ourpcap_read_bpf;
#endif

#ifdef LINUX
	p->read_op =  ourpcap_read_linux;
#endif

	for (;;) {
		if (p->sf.rfile != NULL) {
			/*
			 * 0 means EOF, so don't loop if we get 0.
			 */
			n = pcap_offline_read(p, cnt, callback, user);
		} else {
			/*
			 * XXX keep reading until we get something
			 * (or an error occurs)
			 */
			do {
				n = p->read_op(p, cnt, callback, user);
			} while (n == 0);
		}
		if (n <= 0)
			return (n);
		if (cnt > 0) {
			cnt -= n;
			if (cnt <= 0)
				return (0);
		}
	}
}


// copy pcap structure, allocate new buffer for each thread.
pcap_t *ourpcap_clone(pcap_t *p)
{
	pcap_t *clone;
	signal_block();
	spinlock(&locks[LOCK_MALLOC]);	

	clone = malloc(sizeof(pcap_t));
	if (!clone)
		error(0, "ourpcap_clone(): error in malloc()\n");
	*clone = *p;
	clone->buffer = malloc(p->bufsize + p->offset);

	if (!clone->buffer) {
		free(clone);
		printf("ourpcap_clone(): error in malloc()\n");
	}

#ifdef _AIX
	// For some strange reason this seems to prevent the EFAULT 
	// problems we have experienced from AIX BPF.
	memset(p->buffer, 0x0, p->bufsize);

#endif
	unlock(&locks[LOCK_MALLOC]);
	signal_unblock();
	return clone;
}

#ifndef LINUX

inline int readwouldblock(int fd)
{
	fd_set rset;
	struct timeval time;
	int rc, maxfd = fd + 1;
	FD_ZERO(&rset);
	FD_SET(fd, &rset);
	// select wait value
	time.tv_sec = 0;
	time.tv_usec = 0;

	if ((rc = select(maxfd, &rset, NULL, NULL, &time)) < 0) {
		return 0;
	} else if (FD_ISSET(fd, &rset)) {
		return 1;
	}
	return 0;
}
volatile static int test;
int ourpcap_read_bpf(pcap_t *p, int cnt, pcap_handler callback, u_char *user)
{
	int fd, v = 1, rc;
	int cc;
	int n = 0;
	register u_char *bp, *ep;
	struct bpf_insn *fcode;
	struct timeval timeout;
//
	fd = pcap_fileno(p);
	/* instruct bpf to not dilly-dally about
	 * waiting for the buffer to fill before read
	 * returns.  We can't do that with two interfaces,
	 * else we risk losing packets.
	 */
//ioctl(fd, BIOCIMMEDIATE, &v);
//
	fcode = p->md.use_bpf ? NULL : p->fcode.bf_insns;

 again:

	// if time to do report then children wait
	// while parent does report:
	if (sync_pending()) {
		// user is NULL for parent
		if (user == NULL) {
			sync_waitforchildren();
			doreport();
			sync_go();
		} else {
			sync_waitforparent();
		}
	}

	/*
	 * Has "pcap_breakloop()" been called?
	 */

	if (p->break_loop) {
		/*
		 * Yes - clear the flag that indicates that it
		 * has, and return -2 to indicate that we were
		 * told to break out of the loop.
		 */

		p->break_loop = 0;
		return (-2);
	}
	cc = p->cc;
	if (p->cc == 0) {

		/* lock kernel buffer(p->fd) because it is a shared resource.
		 * Each one has its own copy of p.
		 */

//		signal_block();
		spinlock(&locks[LOCK_KERNBUF]);
/*		
		if (readwouldblock(fd) == 1) {
			unlock(&locks[LOCK_KERNBUF]);
			signal_unblock();
			printf("would block\n");
			goto again;
		}
*/

//		printf("locked, before read:%d, %d\n", user, test);
		cc = read(p->fd, (char *)p->buffer, p->bufsize);
	/*
		ioctl(fd, FIONREAD, &rc);
		if (rc) {
			cc = read(p->fd, (char *)p->buffer, p->bufsize);
		} else {
			unlock(&locks[LOCK_KERNBUF]);
			return 0;
		}
	*/
//		printf("unlocking, after read:%d, %d\n", user, test);
		unlock(&locks[LOCK_KERNBUF]);
//		signal_unblock();


		if(debug) {
			printf("our_pcap_read_bpf: post read: cc %d, %d\n", getpid(), cc);
		}

		if (cc < 0) {
			/* Don't choke when we get ptraced */
			switch (errno) {

			case EINTR:
				printf("EINTR\n");
				goto again;

#ifdef _AIX
			case EFAULT:
				printf("EFAULT\n");

				/*
				 * Sigh.  More AIX wonderfulness.
				 *
				 * For some unknown reason the uiomove()
				 * operation in the bpf kernel extension
				 * used to copy the buffer into user 
				 * space sometimes returns EFAULT. I have
				 * no idea why this is the case given that
				 * a kernel debugger shows the user buffer 
				 * is correct. This problem appears to 
				 * be mostly mitigated by the memset of 
				 * the buffer before it is first used. 
				 * Very strange.... Shaun Clowes
				 *
				 * In any case this means that we shouldn't 
				 * treat EFAULT as a fatal error; as we
				 * don't have an API for returning
				 * a "some packets were dropped since
				 * the last packet you saw" indication,
				 * we just ignore EFAULT and keep reading.
				 */
				goto again;
#endif 
  
			case EWOULDBLOCK:
				printf("EWOULDBLOCK\n");
				goto again;

#if defined(sun) && !defined(BSD)
			/*
			 * Due to a SunOS bug, after 2^31 bytes, the kernel
			 * file offset overflows and read fails with EINVAL.
			 * The lseek() to 0 will fix things.
			 */
			case EINVAL:

				/*locking the kernel buffer*/
				spinlock(&locks[LOCK_KERNBUF]);
				if (lseek(p->fd, 0L, SEEK_CUR) +
				    p->bufsize < 0) {
					(void)lseek(p->fd, 0L, SEEK_SET);
					unlock(&locks[LOCK_KERNBUF]);
					goto again;
				}
				/* fall through */
				unlock(&locks[LOCK_KERNBUF]);
#endif
			}
			snprintf(p->errbuf, PCAP_ERRBUF_SIZE, "read: %s",
			    pcap_strerror(errno));
			printf(p->errbuf);


			return (-1);
		}
		bp = p->buffer;
	} else { 
		bp = p->bp;  /*no error only 1 line*/
	}

	/*
	 * Loop through each packet.
	 */
#define bhp ((struct bpf_hdr *)bp)
	ep = bp + cc;
	while (bp < ep) {
		register int caplen, hdrlen;

		/*
		 * Has "pcap_breakloop()" been called?
		 * If so, return immediately - if we haven't read any
		 * packets, clear the flag and return -2 to indicate
		 * that we were told to break out of the loop, otherwise
		 * leave the flag set, so that the *next* call will break
		 * out of the loop without having read any packets, and
		 * return the number of packets we've processed so far.
		 */
		if (p->break_loop) {
			if (n == 0) {
				p->break_loop = 0;
				return (-2);
			} else {
				p->bp = bp;
			 	p->cc = ep - bp;
				return (n);
			}
		}

		caplen = bhp->bh_caplen;
		hdrlen = bhp->bh_hdrlen;
		/*
		 * Short-circuit evaluation: if using BPF filter
		 * in kernel, no need to do it now. second time.. same thing
		 * in our case fcode ==null and bpf filter has been applied in the kernel itself..
		 */
		if (fcode == NULL ||
			bpf_filter(fcode, bp + hdrlen, bhp->bh_datalen, caplen)) { 
#ifdef _AIX
			/*
			 * AIX's BPF returns seconds/nanoseconds time
			 * stamps, not seconds/microseconds time stamps.
			 *
			 * XXX - I'm guessing here that it's a "struct
			 * timestamp"; if not, this code won't compile,
			 * but, if not, you want to send us a bug report
			 * and fall back on using DLPI.  It's not as if
			 * BPF used to work right on AIX before this
			 * change; this change attempts to fix the fact
			 * that it didn't....
			 */
			bhp->bh_tstamp.tv_usec = bhp->bh_tstamp.tv_usec/1000;
#endif
			/*
			 * XXX A bpf_hdr matches a pcap_pkthdr.
			 */
			(*callback)(user, (struct pcap_pkthdr*)bp, bp + hdrlen);
			bp += BPF_WORDALIGN(caplen + hdrlen);
			if (++n >= cnt && cnt > 0) {
				p->bp = bp;
				p->cc = ep - bp;
				return (n);
			}
		} else {
			/*
			 * Skip this packet. bpf says this is not a pkt we want 
			 */
			bp += BPF_WORDALIGN(caplen + hdrlen);
		}
	}
#undef bhp
	p->cc = 0;
	return (n);
}

#endif //#ifndef LINUX


#ifdef LINUX

#define  pcap_snprintf snprintf 
extern int snprintf (char *, size_t, const char *, ...);

/*
 *  Read a packet from the socket calling the handler provided by
 *  the user. Returns the number of packets received or -1 if an
 *  error occured.
 */
int ourpcap_read_linux(pcap_t *handle, int cnt, pcap_handler callback,
	u_char *userdata)
{
	int fd, rc;
	u_char			*bp;
	int			offset; 	
#ifdef HAVE_PF_PACKET_SOCKETS
	struct sockaddr_ll	from;
	struct sll_header	*hdrp;
#else
	struct sockaddr		from;
#endif
	socklen_t		fromlen;
	int			packet_len, caplen;
	struct pcap_pkthdr	pcap_header;

	fd = pcap_fileno(handle);

#ifdef HAVE_PF_PACKET_SOCKETS
	/*
	 * If this is a cooked device, leave extra room for a
	 * fake packet header.
	 */
	if (handle->md.cooked)
		offset = SLL_HDR_LEN;
	else
		offset = 0;
#else
	/*
	 * This system doesn't have PF_PACKET sockets, so it doesn't
	 * support cooked devices.
	 */
	offset = 0;
#endif

	/* Receive a single packet from the kernel */
	if(debug){
	  printf("Inside our_pcap_read_packet\n");
	}

	bp = handle->buffer + handle->offset;
	do {
		// if time to do report then children wait
		// while parent does report:
		if (sync_pending()) {
			// user is NULL for parent
			if (userdata == NULL) {
				sync_waitforchildren();
				doreport();
				sync_go();
	
			} else {
				sync_waitforparent();
			}
		}
		/*
		 * Has "pcap_breakloop()" been called?
		 */
		if (handle->break_loop) {
			/*
			 * Yes - clear the flag that indicates that it
			 * has, and return -2 as an indication that we
			 * were told to break out of the loop.
			 */
			handle->break_loop = 0;
			return -2;
		}
		fromlen = sizeof(from);

		/* lock kernel buffer(p->fd) because it is a shared resource.
		 * Each one has its own copy of p.
		 */

		spinlock(&locks[LOCK_KERNBUF]);
		ioctl(fd, FIONREAD, &rc);
		if (rc) {
			packet_len = recvfrom(
				handle->fd, bp + offset,
				handle->bufsize - offset, MSG_TRUNC,
				(struct sockaddr *) &from, &fromlen);
		} else {
			unlock(&locks[LOCK_KERNBUF]);
			return 0;
		}
		unlock(&locks[LOCK_KERNBUF]);

		if (errno == EINTR) {
			printf("Interupted: %d\n", getpid());
		}
	} while (packet_len == -1 && errno == EINTR);


	/* Check if an error occured */
	if (packet_len == -1) {
		if (errno == EAGAIN)
			return 0;	/* no packet there */
		else {
                        spinlock(&locks[LOCK_MALLOC]);
			snprintf(handle->errbuf, sizeof(handle->errbuf),
				 "recvfrom: %s", pcap_strerror(errno));
			unlock(&locks[LOCK_MALLOC]);
			return -1;
		}
	}

#ifdef HAVE_PF_PACKET_SOCKETS
	/*
	 * If this is from the loopback device, reject outgoing packets;
	 * we'll see the packet as an incoming packet as well, and
	 * we don't want to see it twice.
	 *
	 * We can only do this if we're using PF_PACKET; the address
	 * returned for SOCK_PACKET is a "sockaddr_pkt" which lacks
	 * the relevant packet type information.
	 */
	if (!handle->md.sock_packet &&
	    from.sll_ifindex == handle->md.lo_ifindex &&
	    from.sll_pkttype == PACKET_OUTGOING)
		return 0;
#endif

#ifdef HAVE_PF_PACKET_SOCKETS
	/*
	 * If this is a cooked device, fill in the fake packet header.
	 */
	if (handle->md.cooked) {
		/*
		 * Add the length of the fake header to the length
		 * of packet data we read.
		 */
		packet_len += SLL_HDR_LEN;

		hdrp = (struct sll_header *)bp;

		/*
		 * Map the PACKET_ value to a LINUX_SLL_ value; we
		 * want the same numerical value to be used in
		 * the link-layer header even if the numerical values
		 * for the PACKET_ #defines change, so that programs
		 * that look at the packet type field will always be
		 * able to handle DLT_LINUX_SLL captures.
		 */
		switch (from.sll_pkttype) {

		case PACKET_HOST:
			hdrp->sll_pkttype = htons(LINUX_SLL_HOST);
			break;

		case PACKET_BROADCAST:
			hdrp->sll_pkttype = htons(LINUX_SLL_BROADCAST);
			break;

		case PACKET_MULTICAST:
			hdrp->sll_pkttype = htons(LINUX_SLL_MULTICAST);
			break;

		case PACKET_OTHERHOST:
			hdrp->sll_pkttype = htons(LINUX_SLL_OTHERHOST);
			break;

		case PACKET_OUTGOING:
			hdrp->sll_pkttype = htons(LINUX_SLL_OUTGOING);
			break;

		default:
			hdrp->sll_pkttype = -1;
			break;
		}

		hdrp->sll_hatype = htons(from.sll_hatype);
		hdrp->sll_halen = htons(from.sll_halen);
		memcpy(hdrp->sll_addr, from.sll_addr,
		    (from.sll_halen > SLL_ADDRLEN) ?
		      SLL_ADDRLEN :
		      from.sll_halen);
		hdrp->sll_protocol = from.sll_protocol;
	}
#endif

	/*
	 * XXX: According to the kernel source we should get the real
	 * packet len if calling recvfrom with MSG_TRUNC set. It does
	 * not seem to work here :(, but it is supported by this code
	 * anyway.
	 * To be honest the code RELIES on that feature so this is really
	 * broken with 2.2.x kernels.
	 * I spend a day to figure out what's going on and I found out
	 * that the following is happening:
	 *
	 * The packet comes from a random interface and the packet_rcv
	 * hook is called with a clone of the packet. That code inserts
	 * the packet into the receive queue of the packet socket.
	 * If a filter is attached to that socket that filter is run
	 * first - and there lies the problem. The default filter always
	 * cuts the packet at the snaplen:
	 *
	 * # tcpdump -d
	 * (000) ret      #68
	 *
	 * So the packet filter cuts down the packet. The recvfrom call
	 * says "hey, it's only 68 bytes, it fits into the buffer" with
	 * the result that we don't get the real packet length. This
	 * is valid at least until kernel 2.2.17pre6.
	 *
	 * We currently handle this by making a copy of the filter
	 * program, fixing all "ret" instructions with non-zero
	 * operands to have an operand of 65535 so that the filter
	 * doesn't truncate the packet, and supplying that modified
	 * filter to the kernel.
	 */

	caplen = packet_len;
	if (caplen > handle->snapshot)
		caplen = handle->snapshot;

	/* Run the packet filter if not using kernel filter */
	if (!handle->md.use_bpf && handle->fcode.bf_insns) {
		if (bpf_filter(handle->fcode.bf_insns, bp,
		                packet_len, caplen) == 0)
		{
			/* rejected by filter */
			return 0;
		}
	}

	/* Fill in our own header data */

	spinlock(&locks[LOCK_KERNBUF]);	
	if (ioctl(handle->fd, SIOCGSTAMP, &pcap_header.ts) == -1) {
		snprintf(handle->errbuf, sizeof(handle->errbuf),
			 "ioctl: %s", pcap_strerror(errno));
		unlock(&locks[LOCK_KERNBUF]);
		return -1;
	}
	unlock(&locks[LOCK_KERNBUF]);
	
	pcap_header.caplen	= caplen;
	pcap_header.len		= packet_len;

	/*
	 * Count the packet.
	 *
	 * Arguably, we should count them before we check the filter,
	 * as on many other platforms "ps_recv" counts packets
	 * handed to the filter rather than packets that passed
	 * the filter, but if filtering is done in the kernel, we
	 * can't get a count of packets that passed the filter,
	 * and that would mean the meaning of "ps_recv" wouldn't
	 * be the same on all Linux systems.
	 *
	 * XXX - it's not the same on all systems in any case;
	 * ideally, we should have a "get the statistics" call
	 * that supplies more counts and indicates which of them
	 * it supplies, so that we supply a count of packets
	 * handed to the filter only on platforms where that
	 * information is available.
	 *
	 * We count them here even if we can get the packet count
	 * from the kernel, as we can only determine at run time
	 * whether we'll be able to get it from the kernel (if
	 * HAVE_TPACKET_STATS isn't defined, we can't get it from
	 * the kernel, but if it is defined, the library might
	 * have been built with a 2.4 or later kernel, but we
	 * might be running on a 2.2[.x] kernel without Alexey
	 * Kuznetzov's turbopacket patches, and thus the kernel
	 * might not be able to supply those statistics).  We
	 * could, I guess, try, when opening the socket, to get
	 * the statistics, and if we can not increment the count
	 * here, but it's not clear that always incrementing
	 * the count is more expensive than always testing a flag
	 * in memory.
	 */
	handle->md.stat.ps_recv++;

	/* Call the user supplied callback function */
	callback(userdata, &pcap_header, bp);

	return 1;
}

#endif // LINUX

/* OLD:
// copy pcap structure, allocate new buffer for each thread.
pcap_t* pcap_clone(pcap_t *p)
{
        char ebuf[PCAP_ERRBUF_SIZE];
	pcap_t *ptemp;

	// block signal so that thread is not interrupted by a signal
	block_signal();

	spinlock(&locks[LOCK_MALLOC]);	
	ptemp = (pcap_t *)malloc(sizeof(pcap_t));

	if (ptemp == NULL) {
		snprintf(ebuf, PCAP_ERRBUF_SIZE, "malloc: %s",
		    pcap_strerror(errno));
		return (NULL);
	}
	
	//ptemp is the same structure except for a new buffer
	*ptemp = *p; 
	
	ptemp->buffer = (u_char *)malloc(p->bufsize); // every thread gets own malloced buffer
	
	if (ptemp->buffer == NULL) {
		snprintf(ebuf, PCAP_ERRBUF_SIZE, "malloc: %s",
		    pcap_strerror(errno));			
		free(ptemp);
		return (NULL);
	}

	unlock(&locks[LOCK_MALLOC]);

	unblock_signal();

#ifdef _AIX
	// For some strange reason this seems to prevent the EFAULT 
	// problems we have experienced from AIX BPF. 
	memset(p->buffer, 0x0, p->bufsize);

#endif

	if(debug){
          printf("Exiting pcap_clone.\n");
        }
	return ptemp;
}
 */
#endif

