/* pktkluge.c
* 
* This is really a general (and stoopid) way to deal with 
* the problem of having a counter that NEEDS to be
* zeroed out at alarm time, but is unavailable.
* Thus it is an attempt to deal with linux libpcap inability to
* allow the caught/dropped stats values to be zeroed
* out periodically, although strangely enough that
# is what the linux kernel does.  We track counters, and
* try to deal with overflow if necessary.
* This is plan A.  plan B could be trying
* to use the linux getsockopt on a packets socket to read these
* things out ourselves and ignore libpcap.
* The packets socket appearently does what is wanted,
* (zero out caught/dropped) but libpcap doesn't give
# us the descriptor that we need, therefore unless
# we do everything ourselves (should?) we can't make
# this work.
*/

#ifdef LINUX

#include <pcap.h>
#include <sys/socket.h>
#include <linux/if_packet.h>

/* caught
*/
#define MAXCOUNTERS 2
static int firsttime[2] = {1,1};
static unsigned int pkts_caught[MAXCOUNTERS] = {0,0};
static unsigned int pkts_previous[MAXCOUNTERS] = {0,0};

/* dropped
*/
static unsigned int pkts_dropped[MAXCOUNTERS] = {0,0};
static unsigned int pkts_dropped_previous[MAXCOUNTERS] = {0,0};

void
pkts_summary(int index,
unsigned int caught, 
unsigned int dropped, 
unsigned int *newcaught, 
unsigned int *newdropped)
{

	if (index < 0 || index > (MAXCOUNTERS-1)) {
		return;
	}

	if (firsttime[index]) {
		pkts_previous[index] = caught;
		pkts_dropped_previous[index] 
			= dropped;
		firsttime[index] = 0;
	}
	else {
		if (caught >= pkts_previous[index]) {
		        *newcaught = 
			caught - pkts_previous[index];
			pkts_previous[index] = caught;
		}
		/* overflow but how many times?
		 * just reinitialize
		*/
		else {
			pkts_previous[index] = caught;
		}

		if (dropped >= pkts_dropped_previous[index]) {
			*newdropped = dropped - pkts_dropped_previous[index];
			pkts_dropped_previous[index] = dropped;
		}
		/* overflow but how many times?
		*/
		else {
			pkts_dropped_previous[index] = dropped;
		}
	}
}

#ifdef PLANBNOGO
/* plan B
 * this doesn't work, because libpcap
 * isn't giving us the right fd.
*/

int
mypcap_stats_linux
(int fd, unsigned int *caught, unsigned int *dropped)
{
	struct tpacket_stats kstats;
	socklen_t len = sizeof (struct tpacket_stats);
	unsigned int r, d;

	/*
	 * Try to get the packet counts from the kernel.
	 */
	if (getsockopt(
		fd,
		SOL_PACKET, 
		PACKET_STATISTICS,
		&kstats, 
		&len) > -1) {
		*caught = kstats.tp_packets;
		*dropped = kstats.tp_drops;
		return 0;
	}
}
#endif

#endif

