#ifdef THREAD
/*
 * Copyright (c) 2007 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// spinlock.c
// spinlock primitives

/*
 * SPINLOCK
 *
 * Cmpxchg
 * Compares the value in the AL, AX, or EAX register (depending on the size of 
 * the operand) with the first operand (destination operand).If the two values 
 * are equal, the second operand (source operand) is loaded into the 
 * destination operand. Otherwise, the destination operand is loaded into 
 * the AL, AX, or EAX register.
 *

 * The cmpxchg (compare and exchange) instruction is available only on the 
 * 80486 and later processors. It supports the following syntax: 
                cmpxchg reg, reg
                cmpxchg mem, reg
 * The operands must be the same size (eight, sixteen, or thirty-two bits). 
 * This instruction also uses the accumulator register; it automatically 
 * chooses al, ax, or eax to match the size of the operands.
 * This instruction compares al, ax, or eax with the first operand(dest operand
 * ) and sets the zero flag if they are equal. Then cmpxchg copies the second 
 * operand into the first. 
 * If they are not equal, cmpxchg copies the first operand into the 
 * accumulator. 
 *
 * i.e. if the expected and actual values are equal, write eax to memory"
 * The source operand is written into the destination operand, not EAX.
 *
 * The �ock cmpxchg�instruction basically stores the locking threads' id 
 * in the object header, so another thread that tries to lock the same 
 * object can see that it� already locked.
 */

#include "spinlock.h"
#include <strings.h>

inline void 
lockinit(lock_t *lock)
{
	bzero(lock, sizeof(lock_t));	
}

inline void 
spinlock(lock_t * lock)
{
	// use 1 for the value of a locked lock
	register unsigned char lockval = 1;
	__asm__ __volatile__(  "1:\t cmpb $0,%2 \n\t"
		"jne 1b     \n"
		"mov $0,%%eax \n"
		"lock; cmpxchgb %1,%0 \n"
		"jnz 1b   \n"
		: "=m" (lock->mutex)
		: "r"(lockval),"m" (lock->mutex)
		: "%eax", "memory");
}

inline void 
unlock(lock_t * lock)
{
	__asm__ __volatile__ ("movb $0,%0;"
		: "=m" (lock->mutex));
}


/* OLD:
// this function needs to be cleaned up.  We no longer care
// who owns the lock at this level:
inline void spinlock(lock_t * lock, unsigned char *owner)
{

#define PARENT_LOCKVAL 1
#define CHILD_LOCKVAL 2

	register unsigned char islocked; 
	if(owner != 0){
		islocked = PARENT_LOCKVAL;
	} else { 
		islocked = CHILD_LOCKVAL;
	}
	__asm__ __volatile__(  "1:\t cmpb $0,%2 \n\t"
		"jne 1b     \n"
		"mov $0,%%eax \n"
		"lock; cmpxchgb %1,%0 \n"
		"jnz 1b   \n"
		: "=m" (*(lock))
		: "r"(islocked),"m" (*(lock))
		: "%eax", "memory");
}
*/

#endif


