#ifdef THREAD
/*
 * Copyright (c) 2007 Portland State University 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Portland State University.
 * 4. The name of Portland State University may not be used to endorse 
 *    or promote products derived from this software without specific 
 *    prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR/S ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR/S BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// sync.h
// This file houses all global variables and locks used to synchronize ourmon

#ifndef SYNC_H
#define SYNC_H

#include "spinlock.h"
#include "thread.h"

static pid_t *pids;
static int numpids = 0;
static thread_t *threads;
volatile static int numthreads = 0;
volatile static int pending;
static lock_t pendinglock;

void inline aquire(lock_t *lock);
void inline release(lock_t *lock);
inline void sync_alert();
inline void sync_go();
inline void sync_waitforchildren();
inline int sync_waitforparent();
inline int isparent();
inline void sync_destroythreads(thread_t *threads, int nthreads);
inline void sync_createthreads(int nthreads, pcap_t *p, int dlt, int cnt);

// locks used to synchronize ourmon:
#define LOCK_SYNCPCAP 0
#define LOCK_KERNBUF 1 
#define LOCK_FIXED_IPPROTO 2	
#define LOCK_FIXED_TCP3   3
#define LOCK_FIXED_IPRANGE1 4
#define LOCK_SPEC_IPPROTO  5
#define LOCK_FIXED_CAST  6
#define LOCK_FIXED_SIZE  7
#define LOCK_TOPN_IP 8
#define LOCK_TOPN_SYN 9
#define LOCK_TOPN_ICMPERR 10
#define LOCK_TOPN_PORT 11
#define LOCK_TOPN_IRC 12
#define LOCK_TOPN_IP_SCANS_INPUT 13
#define LOCK_TOPN_PORTS_SCANS_INPUT 14
#define LOCK_REPORTWRITE 15
#define LOCK_MALLOC 16
#define LOCK_IPMESH 17
#define LOCK_RING 18
#define LOCK_TRIGGER 19
#define LOCK_BLIST   20
#define LOCK_ELGRANDE   21
#define NUMLOCKS 22

lock_t locks[NUMLOCKS];

// where is this used?:
#define MAXBPFSPECS	100	/* max. number of labels for all bpf graphs */
lock_t bpflocks[MAXBPFSPECS];

#endif
#endif
