#!/bin/sh
# 
# do hourly ourmon summarization
#
# first run top N flows script
# then run top ip src, ip dst script
#
# variables overwritten at config time include
#	1. BASEDIR
#
# everything else is relative to OURPATH

BASEDIR=/home/mrourmon
OURPATH=$BASEDIR

OURBIN=$OURPATH/bin
WEBDIR=$OURPATH/web.pages
LOGDIR=$OURPATH/logs
BASELOGFILE="$OURPATH/logs/topn_today/topn_ip.log"
BASEICMPFILE="$OURPATH/logs/topn_today/topn_icmp.log"
BASEUDPFILE="$OURPATH/logs/topn_today/topn_udp.log"
BASEPKTSFILE="$OURPATH/logs/topn_today/topn_ip_pkts.log"
BASESYNFILE="$OURPATH/logs/topn_today/syn_list.log"
BASEWORMDIR="$OURPATH/logs/tworm/tworm_today"
BASEP2PDIR="$OURPATH/logs/rawp2p/rawp2p_today"
BASESYNDUMPDIR="$OURPATH/logs/rawsyndump/rawsyndump_today"
BASEEMAILDUMPDIR="$OURPATH/logs/rawemail/rawemail_today"
BASESYNWFILE="$OURPATH/logs/topn_today/syn_worm.log"
BASEPOTDUMPDIR="$OURPATH/logs/rawpotdump/rawpotdump_today"

OMBATCHIP=$OURBIN/ombatchip.pl 
OMBATCHIPSRC=$OURBIN/ombatchipsrc.pl 
OMBATCHSYN=$OURBIN/ombatchsyn.pl 

IRC=$OURBIN/irc.pl
TCPWORM=$OURBIN/tcpworm.pl

# acquire homenetwork definition if any
# if back-end config info exists load it
# OURNET is either not defined or defined
# as a possible input parameter with args
# homeconfig.txt must agree with omupdate.pl filename
OURNET=
if [ -f $LOGDIR/homeconfig.txt ]; then
 	OURNET=`cat $LOGDIR/homeconfig.txt`
 	OURNET="-h $OURNET" 
fi

# do top of top flow work
$OMBATCHIP other $BASELOGFILE $WEBDIR/topn_today.txt
$OMBATCHIP icmp $BASEICMPFILE $WEBDIR/topicmp_today.txt
$OMBATCHIP other $BASEUDPFILE $WEBDIR/topudp_today.txt
$OMBATCHIP pkts $BASEPKTSFILE $WEBDIR/topnippkts_today.txt

# do syn analysis
if [ -f $BASESYNFILE ]; then
	$OMBATCHSYN -N $BASESYNFILE $WEBDIR/topsyn_today.txt
fi
if [ -f $BASESYNWFILE ]; then
	$OMBATCHSYN -W 100 -N -c 1000 $BASESYNWFILE $WEBDIR/allworm_today.txt
fi

# do top src/dst work
if [ -f $BASELOGFILE ]; then
	$OMBATCHIPSRC $BASELOGFILE $WEBDIR/topn_ipsrc_today.txt $WEBDIR/topn_ipdst_today.txt
fi
if [ -f $BASEICMPFILE ]; then
	$OMBATCHIPSRC $BASEICMPFILE $WEBDIR/topicmp_ipsrc_today.txt $WEBDIR/topicmp_ipdst_today.txt
fi
if [ -f $BASEUDPFILE ]; then
	$OMBATCHIPSRC $BASEUDPFILE $WEBDIR/topudp_ipsrc_today.txt $WEBDIR/topudp_ipdst_today.txt
fi

# hourly irc summarization report
# to modify for automated IRC blacklist add these parameters
# assuming that ourshadow is a db file in the proper format
# -B /home/mrourmon/etc/ourshadow -e $WEBDIR/event_today.txt
$IRC -d  $OURPATH/logs/rawirc/rawirc_today > $WEBDIR/ircreport_today.txt

# do combined log analysis (top "busy" ip addresses in multiple kinds of logs)
# release this later

# hourly tcpworm.pl summarization for tcpworm.txt files

# 4 kinds of hourly/daily summarization for tcpworm
# 1. all
# 2. 445 worms 
# TBD index.html and batchipall.sh still need work for these two
# 2.5 home net 445 worms 
# 2.6 home net 80 mentioning worms at home
# 3. only weight 40 and above
# 4. p2p summarization only, not worm

# all unfiltered
$TCPWORM -i -z $BASEWORMDIR > $WEBDIR/wormsum.all_daily.txt

# port 445 only, weight 20 or above
$TCPWORM -i -w 20 -4 445 -z $BASEWORMDIR > $WEBDIR/wormsum.445_daily.txt

# port 445 only, weight 20 or above, home addresses only
if [ "$OURNET" ]; then
$TCPWORM $OURNET -i -w 20 -4 445 -z $BASEWORMDIR > $WEBDIR/wormsum.home.445_daily.txt
fi

# port 80 mentioned, sort by instances
if [ "$OURNET" ]; then
$TCPWORM $OURNET -i -w 20 -4 80 -z $BASEWORMDIR > $WEBDIR/wormsum.home.80_daily.txt
fi

# all ports, weight 40 or above
$TCPWORM -i -w 40  -z $BASEWORMDIR > $WEBDIR/wormsum.40w_daily.txt

# p2p summarization
$TCPWORM -t -z $BASEP2PDIR > $WEBDIR/p2psum.daily.txt

# syndump summarization
$TCPWORM -t -z $BASESYNDUMPDIR > $WEBDIR/syndump.daily.txt

# email summarization.  
$TCPWORM -e 100 -z $BASEEMAILDUMPDIR > $WEBDIR/emaildump.daily.txt

# potdump summarization
$TCPWORM -i -z $BASEPOTDUMPDIR > $WEBDIR/potdump.daily.txt
