#! /usr/bin/perl

#
# daily.pl
#
# syntax:
#	# daily.pl base_log_dir
# e.g.,
#	# daily.pl /home/mrourmon/logs
#
# run at around midnight to start log directory over for next day.
# run from crontab.
# 0	0	*	*	*	root	/home/mrourmon/bin/daily.pl /home/mrourmon/logs
#
# All log dirs are found under a base log directory with the following dir scheme:
#
# /home/mrourmon/logs
#	mon.lite
#	tworm
#	topn
#	portreport
#	p2preport
#	rawirc
#	irc
#	udpreport
#	rawpotdump
#
#	each directory above has Sun Mon Tue ... Sat daily directories
#
# This perl script is called once at midnight to prepare for the next
# day's logging.
#
# it's job is to zero out the daily (Sun Mon Tue Wed Thu Fri Sat)
# directory in the base logging directory for all 3 of the above.
#
# for the topn logging directory it also creates a symlink
# to a cur for the current day.
#
# Assumption: output_dir has subdirs for days of week.  Mon, Tue, etc.
# Thus output format is:  log/Mon/date.mon.lite
# form is thus:
# Sun Mon Tue Wed Thu Fri Sat 

$debug = 0;
$argc=$#ARGV+1;

if ($argc != 1) {
	print "syntax: # daily.pl output_dir\n";
	print "example: # daily.pl /home/mrourmon/logs\n";
	exit(1);
}
use File::Basename();

use Time::Local;
# comment in for ctime, note that it overrides localtime
use Time::localtime;

my $outputdir = $ARGV[0];

################################################### 
$start_time=`date`;
$start_time=~ s/\n//g;		# zap the newline at the end of the date
@time = split(/ /, $start_time);
$dow = $time[0];

chdir($outputdir);

# zap the event database so we start over */
system("rm -fr eventdb.db");   

# mon.lite daily prep
system("rm -fr mon.lite/$dow"); 
system("rm -fr mon.lite/mon_today"); 
system("mkdir mon.lite/$dow"); 
system("ln -s $dow mon.lite/mon_today");

# tworm daily prep
system("rm -fr tworm/$dow"); 
system("rm -fr tworm/tworm_today"); 
system("mkdir tworm/$dow"); 
system("ln -s $dow tworm/tworm_today");

# topn daily prep
system("rm -fr $dow"); 
system("rm -f topn_today"); 
system("mkdir $dow"); 
system("ln -s $dow topn_today");

# portreport daily prep
system("rm -fr portreport/$dow"); 
system("rm -f portreport/portreport_today"); 
system("mkdir portreport/$dow"); 
system("ln -s $dow portreport/portreport_today");

# p2preport daily prep
system("rm -fr p2preport/$dow"); 
system("rm -f p2preport/p2preport_today"); 
system("mkdir p2preport/$dow"); 
system("ln -s $dow p2preport/p2preport_today");

# rawirc daily prep
system("rm -fr rawirc/$dow"); 
system("rm -f rawirc/rawirc_today"); 
system("mkdir rawirc/$dow"); 
system("ln -s $dow rawirc/rawirc_today");

# irc daily prep
system("rm -fr irc/$dow"); 
system("rm -f irc/irc_today"); 
system("mkdir irc/$dow"); 
system("ln -s $dow irc/irc_today");

# udpreport daily prep
system("rm -fr udpreport/$dow"); 
system("rm -f udpreport/udpreport_today"); 
system("mkdir udpreport/$dow"); 
system("ln -s $dow udpreport/udpreport_today");

# psuportreport daily prep
#system("rm -fr psuportreport/$dow"); 
#system("rm -f psuportreport/portreport_today"); 
#system("mkdir psuportreport/$dow"); 
#system("ln -s $dow psuportreport/portreport_today");

# rawp2p daily prep
system("rm -fr rawp2p/$dow"); 
system("rm -f rawp2p/rawp2p_today"); 
system("mkdir rawp2p/$dow"); 
system("ln -s $dow rawp2p/rawp2p_today");

# full/rawsynreport daily prep
system("rm -fr rawsyndump/$dow");
system("rm -f rawsyndump/rawsyndump_today");
system("mkdir rawsyndump/$dow");
system("ln -s $dow rawsyndump/rawsyndump_today");

# full/rawemail daily prep
system("rm -fr rawemail/$dow");
system("rm -f rawemail/rawemail_today");
system("mkdir rawemail/$dow");
system("ln -s $dow rawemail/rawemail_today");

# full/email daily prep
#system("rm -fr email/$dow");
#system("rm -f email/email_today");
#system("mkdir email/$dow");
#system("ln -s $dow email/email_today");

# full/rawipmesh daily prep
#system("rm -fr rawipmesh/$dow");
#system("rm -f rawipmesh/rawipmesh_today");
#system("mkdir rawipmesh/$dow");
#system("ln -s $dow rawipmesh/rawipmesh_today");

# full/rawpotdump daily prep
system("rm -fr rawpotdump/$dow");
system("rm -f rawpotdump/rawpotdump_today");
system("mkdir rawpotdump/$dow");
system("ln -s $dow rawpotdump/rawpotdump_today");

exit(0);

# end of main
