#! /usr/bin/perl

#
# omupdate.pl - update rrds and create graphs
#
# For syntax:
#	omupdate.pl -h
#	Or see printUsageAndExit below
#
# For defaults (variable values):
#	omupdate.pl -z
#
# input file:
#       mon.lite  - file which is output of ourmon front-end binary
#       	note: mon.lite specifies individual filters by name
#           
# outputs:       
#       will create rrd file for a given filter if it does not
#               exist
#       will update rrd file and create graphics as appropriate
#	will update ourmon "syslog" like log files if logging is on.
#
# dependencies:
#	rrdtool must be installed.
#                
# note:     
#       30 second time delta is assumed throughout
#           
# IMPORTANT: you must update certain default config variables
#	to be correct before running this script.
# OR
#	make sure you have a correct script file with
#	runtime params set correctly. -m, etc.  See the
#	usage message.

# set rrdtool perl lib path if not globally available
# comment this out if globally available
use lib qw( /usr/local/rrdtool-1.0.49/lib/perl ../lib/perl );


# use rrdtool perl lib
use RRDs;

##########################################################
# default configuration variables and values

$tstart = time();

# base directory where rrds live 
my $rrdbase = "/home/mrourmon/rrddata";


# log directory.  we make the assumption here
# that this directory EXISTS and that we have
# one week worth of log backups in it with each
# day a named directory in UNIX date style,
# Sun, Mon, Tue, etc.
# the above is made by the configure script 
my $logbase = "/home/mrourmon/logs";

# set mon.lite input file - this is where it is found
my $datafile = "/home/mrourmon/tmp/mon.lite";

# set base directory for web page output, web files made here
#my $webpath = "/usr/home/mrourmon/web.pages";
my $webpath = "/var/www/htdocs/ourmon";

# dir path to udpreport.txt file
# which by default means it is inside the webpath
my $udpreport_base = ".";

# path of C topn program named drawtopn

# leave this alone
# on Linux, drawtopn is not well-behaved for doing dns in legends.
# best if we move that behavior to omupdate.pl in the next release
# anyway
#
my $dns_lookup = 0;    # 2 is dns on in legends, 0 is off, 1 is not a good idea.

# ourmon syslog style logging defaults to on.
#	
my $sysname;
my $logging="on";		# logging to ourmon-style syslog is on

# There is a very primitive capability for one kind of 
# alert (IP flow too big).
#
# if logging is enabled, we "send" an alert to the alert log file
# for any flows greater than the following:
# sending an alert at the moment simply means logging an entry.
# This is only used with the topn_ip_update function (IP, not TCP
# and not UDP).  It is a hack.
#
# maxalertsize set to 10mbits
my $maxalertsize = 10000000;

# udp threshold weight used with udpreport and associated
#	event log event
my $maxudpweight = 10000000;	# default udp weight

my $topndebug = 0;

# now process command line arguments
# which may change the above configuration variables
processCmdArgs();

# where the bar graph files live
my $barname = "bard";
my $barpath = "$webpath/$barname";

# make sure barpath dir exists
# we put topn bar images in this directory
if (!-d $barpath) {
	mkdir($barpath);
}

if ($logging eq "on") {
	$logging = 1;
}
else {
	$logging = 0;
}

#dumpArgs();
#exit 0;

###################################################
# everything below is NOT configurable

# put time log in same directory as above
my $timelog = "$webpath/time.log";

# store current time time.log file
open(tfd, ">$timelog") || die "cannot open $timelog: $!\n";
$start_time=`date`;
$start_time=~ s/\n//g;
print tfd "$start_time\n";
close(tfd);

# cp datafile to webpath so we can find it
system("cp $datafile $webpath");

# cd to webpath where we create files

chdir($webpath);

#  set rrd file names, error log file name, top N png file
#  name.  rrds and error log must have rrdbase as path
######################################################

my $pkts=$rrdbase . "\/" . "pkts.rrd";
my $ipproto=$rrdbase . "\/" . "fixed_ipproto.rrd";
my $tcp3=$rrdbase . "\/" . "fixed_tcp3.rrd";
my $iprange1=$rrdbase . "\/" . "fixed_iprange1.rrd";
my $cast=$rrdbase . "\/" . "fixed_cast.rrd";
my $l2proto=$rrdbase . "\/" . "fixed_l2proto.rrd";
my $tcpp2p_proto=$rrdbase . "\/" . "tcpp2p_proto.rrd";
my $size=$rrdbase . "\/" . "fixed_size.rrd";
my $errlog=$rrdbase . "\/" . "ourmon.log";
my $flowcount=$rrdbase . "\/" . "flowcount.rrd";
my $twormcount=$rrdbase . "\/" . "twormcount.rrd";
my $irccount=$rrdbase . "\/" . "irc.rrd";
my $netwwcount=$rrdbase . "\/" . "netww.rrd";
my $topnstat=$rrdbase . "\/" . "topnstat.rrd";
my $udpweight=$rrdbase . "\/" . "udpweight.rrd";
my $dnsstats=$rrdbase . "\/" . "dnsstats.rrd";

$ip_flowcount = $udp_flowcount = $tcp_flowcount = $icmp_flowcount = 0;
$topn_exists = 0; 

# get current time 
my $t=time;

# open log file for errors
open(ERR, ">>$errlog") || die "can't open $errlog: $!\n";

# read data from the $datafile, put into @phrases

$delimiter = "\n";
open(FILE, $datafile) || die "can't open mon.lite file: $!\n";
@FILE = <FILE>;
close(FILE);

$start=time;

$phrases = join("<br>",@FILE);

##  fetch each line from datafile, line is separated by "\n"

@phrases = split(/$delimiter/,$phrases);

# walk phrases and process on line by line basis
# one subroutine per filter type


foreach my $elem (@phrases) {

     # get rid off blanks, return, line breaks

     $elem =~ s/\n//g;
     $elem =~ s/<br>//g;
     $saved_line = $elem;	# no blanks removed on this one
     $elem =~ s/ //g;    
     # convert greater than to zilch, until we figure out
     # how to escape it
     $global_line = $elem;
     $global_line=~ s/\>//g;

     # save each item to array @items_in_eachline, from each line,

     @items_in_eachline = split(/:/,$elem);
#     print $items_in_eachline[0],"\n";    

     # check the first item in each line, against the key words, to see 
     #     if filter is enabled.

     # check to see if auto-backend-config needed for home network
     # config info kept in logs/homeconfig.txt
     #
     # We only change the file if the contents has changed.
     # this is not perfect in terms of preventing possible race
     # conditions with batchip.sh BUT it is better than nothing
     # We create the file if it didn't exist before
     # case 1: file does not exist
     # case 2: file exists, but no change
     # case 3: new data needs to be stored

     if ($items_in_eachline[0] eq "#HOMENETWORK") {
	my $network = $items_in_eachline[1];
	my $homefile="$logbase/homeconfig.txt";
	my $writeflag = 0;
	# open the file, note if it does not exist
	unless (open(HANDLE, $homefile)) {
		# does not exist
		$writeflag = 1;  # case 1: file does not exist
	}
	# case 2 possible: file does exist, compare new with previous
	if ($writeflag == 0) {
		my $line = <HANDLE>;	# read contents
		chomp ($line);          # get rid of newline
		# case 2 complete: file exists but no change, bail
		if ($line eq $network) {
			close(HANDLE);
			next;
		}
		# contents changed, we need a write
		$writeflag = 1;
	}
	close(HANDLE);
	# case 3: we need to write the output file
	if ($writeflag) {
		open(tfd, ">$homefile") || die "cannot open $homefile: $!\n";
		print tfd "$network\n";
		close(tfd);
	}
     }
     # sysname makes pkts unique per system with per-system label
     # front-end arranges to make sure this label comes first in
     # mon-lite file.
     elsif ($items_in_eachline[0] eq "sysname") {
	$sysname = $items_in_eachline[1];
	# override rrd name with new name
	$pkts=$rrdbase . "\/" . $sysname . "\." .  "pkts.rrd";
     }
     elsif ($items_in_eachline[0] eq "pkts") {
	unless (open(HANDLE, $pkts)) {
		pkts_create();
	};	
	close(HANDLE);
	pkts_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "dnsstats") {
	unless (open(HANDLE, $dnsstats)) {
		dnsstats_create();
	};	
	close(HANDLE);
	dnsstats_update(@items_in_eachline);
     }
     elsif  ($items_in_eachline[0] eq "fixed_ipproto") {
	unless (open(HANDLE, $ipproto)) {
		ipproto_create();
	};	
	close(HANDLE);
        ipproto_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "fixed_tcp3") {
	unless (open(HANDLE, $tcp3)) {
		tcp3_create();
	};	
	close(HANDLE);
        tcp3_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "fixed_iprange1") {
	unless (open(HANDLE, $iprange1)) {
		iprange1_create();
	};	
	close(HANDLE);
        iprange1_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "fixed_cast") {
	unless (open(HANDLE, $cast)) {
		cast_create();
	};	
	close(HANDLE);
        cast_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "fixed_l2proto") {
	unless (open(HANDLE, $l2proto)) {
		l2proto_create();
	};	
	close(HANDLE);
        l2proto_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "tcpp2p") {
	unless (open(HANDLE, $tcpp2p_proto)) {
		tcpp2p_create();
	};	
	close(HANDLE);
        tcpp2p_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "fixed_size") {
	unless (open(HANDLE, $size)) {
		size_create();
	};	
	close(HANDLE);
        size_update(@items_in_eachline);
     }
     elsif  ($items_in_eachline[0] eq "bpf") {
        bpf_update($saved_line);
     }

     # topn ip (all flows)	
     elsif ($items_in_eachline[0] eq "topn_ip") {
        $ip_flowcount = $items_in_eachline[1] ;
	$topn_exists = 1;
      	topn_bar(\@items_in_eachline);
        $global_line=~ s/\(/[/g;
        $global_line=~ s/\)/]/g;
	if ($logging) {
		omsyslog($logbase, "topn_ip.log", $global_line);
	}
     }
     # topn tcp flows
     elsif ($items_in_eachline[0] eq "topn_tcp") {
        $tcp_flowcount = $items_in_eachline[1] ;
      	topn_bar(\@items_in_eachline);
	if ($logging) {
		omsyslog($logbase, "topn_tcp.log", $global_line);
	}
     }
     # topn udp flows
     elsif ($items_in_eachline[0] eq "topn_udp") {
        $udp_flowcount = $items_in_eachline[1] ;
      	topn_bar(\@items_in_eachline);
	if ($logging) {
		omsyslog($logbase, "topn_udp.log", $global_line);
	}
     }
    elsif ($items_in_eachline[0] eq "topn_icmp") {            
        $icmp_flowcount = $items_in_eachline[1] ;
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "topn_icmp.log", $global_line);
        }                                                
     }                                                  
     elsif ($items_in_eachline[0] eq "topn_ip_pkts") {
        $ip_flowcount = $items_in_eachline[1] ;
      	topn_bar(\@items_in_eachline);
        $global_line=~ s/\(/[/g;
        $global_line=~ s/\)/]/g;
	if ($logging) {
		omsyslog($logbase, "topn_ip_pkts.log", $global_line);
	}
     }
     elsif ($items_in_eachline[0] eq "tcp_ports") {
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "tcp_ports.log", $global_line);
        }                                                
     }
     elsif ($items_in_eachline[0] eq "udp_ports") {
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "udp_ports.log", $global_line);
        }                                                
     } 
     elsif ($items_in_eachline[0] eq "syn_list") {
        if ($logging) {                                    
		omsyslog($logbase, "syn_list.log", $global_line);
        }                                                
      	topn_bar(\@items_in_eachline);
     } 
     # ircstats
     elsif ($items_in_eachline[0] eq "ircstats") {
        irc_update(@items_in_eachline);
     } 
     # tworm is RRDTOOL-based and is also a side-effect of the top syn list
     #
     elsif ($items_in_eachline[0] eq "tworm") {
        tworm_update(@items_in_eachline);
     } 
     elsif ($items_in_eachline[0] eq "icmperror_list") {
        if ($logging) {                                    
		omsyslog($logbase, "icmperror_list.log", $global_line);
        }                                                
      	topn_bar(\@items_in_eachline);
     } 
     elsif ($items_in_eachline[0] eq "udperror_list") {
        if ($logging) {                                    
		omsyslog($logbase, "udperror_list.log", $global_line);
        }                                                
      	topn_bar(\@items_in_eachline);
     } 
     # RRDTOOL-based
     elsif ($items_in_eachline[0] eq "hs_stat") {
        topn_stat(@items_in_eachline);
     }
     # RRDTOOL-based
     elsif ($items_in_eachline[0] eq "netww") {
        netww_update(@items_in_eachline);
     }
     elsif ($items_in_eachline[0] eq "ip_scan") {
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "ip_scan.log", $global_line);
        }                                                
     }
     elsif ($items_in_eachline[0] eq "ip_portscan") {
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "ip_portscan.log", $global_line);
        }                                                
     }
     elsif ($items_in_eachline[0] eq "tcp_portscan") {
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "tcp_portscan.log", $global_line);
        }                                                
     }
     elsif ($items_in_eachline[0] eq "udp_portscan") {
      	topn_bar(\@items_in_eachline);
        if ($logging) {                                    
		omsyslog($logbase, "udp_portscan.log", $global_line);
        }                                                
     }
     elsif ($items_in_eachline[0] eq "elog") {
	# just zap the elog: tag and leave the rest
	# do NOT split on colon.  
        $saved_line =~ s/elog://g;
	storeEvent($eventfile, "ourmon front-end event:", $saved_line);				
     }
	
     # else just ignore it so that it can be tested
	
}  # end loop

if ($topn_exists) {
        $flow_data = "$t:" . $ip_flowcount . ":" . $tcp_flowcount . ":" . $udp_flowcount . ":" .  $icmp_flowcount;
        #print $flow_data;
        flowcount_update($flow_data);
}


close ERR;

$tend = time();

#use this to find out what is going on with elapsed time
#storeEvent($eventfile, "ourmon back-end elapsed time:", $tend-$tstart);				

$esecs = $tend - $tstart;
if ($esecs > 30) {
	storeEvent($eventfile, "ourmon back-end elapsed time too long:", $tend-$tstart);				
	$esecs = 30;
}
exit($esecs);		# return time elapsed in seconds

# end of main

####################################################################

#    update and make png file for pkt capture
#    note: we support two possible interfaces at the same time
#################################################################
sub pkts_update 
{ 
my $graph = "pkts";
if ($sysname) {
	$graph = $sysname . "\." . $graph;
}

# TBD. update event log if pkts are lost
# do update
RRDs::update $pkts, "$t:"."$_[2]".":"."$_[4]".":"."$_[6]".":"."$_[8]";
if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$pkts': $ERROR\n";
      $time=`date`;
      $time=~ s/\n//g;
      print ERR "$time, error with rrd/update on $pkts: $ERROR\n";
};

# args are 1. filename,  2. title,  3. -s time back from now
#
graph_pkts("$graph.daily-png", "daily: pkts captured/dropped"." : ".$start_time, "-2000m");
graph_pkts("$graph.weekly-png", "weekly: pkts capt/dropped"." : ".$start_time, "-12000m");
graph_pkts("$graph.monthly-png", "monthly: pkts capt/dropped"." : ".$start_time, "-800h");
graph_pkts("$graph.yearly-png", "yearly: pkts capt/dropped"." : ".$start_time, "-400d");

}

# spit out rrdtool-speak to graph the PKTS 4 graphs

sub graph_pkts
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "pkts/sec",
"-w", "500",
"-h",  "100", 
"-b", "1024", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
"DEF:cap1=$pkts:cap1:AVERAGE",
"CDEF:pcap1=cap1,30,/",
"DEF:drop1=$pkts:drop1:AVERAGE",
"CDEF:pdrop1=drop1,30,/",
"DEF:cap2=$pkts:cap2:AVERAGE",
"CDEF:pcap2=cap2,30,/",
"DEF:drop2=$pkts:drop2:AVERAGE",
"CDEF:pdrop2=drop2,30,/",
"LINE1:pcap1#0000ff:Capt(1)",
"LINE1:pdrop1#00ff00:Drop(2)",
"LINE1:pcap2#ff0000:Capt(2)",
"LINE1:pdrop2#ff00ff:Drop(2)",
#"COMMENT:               \\r",
"COMMENT:\\s",
"COMMENT:\\s",
"GPRINT:pcap1:MAX:Max capt1 %9.0lf %s",
"GPRINT:pcap1:AVERAGE:Average capt1 %9.0lf %s", 
"GPRINT:pcap1:LAST:Current capt1 %9.0lf %s\\r", 

"GPRINT:pdrop1:MAX:Max drop1 %9.0lf %s",
"GPRINT:pdrop1:AVERAGE:Average drop1 %9.0lf %s", 
"GPRINT:pdrop1:LAST:Current drop1 %9.0lf %s\\r", 

"GPRINT:pcap2:MAX:Max capt2 %9.0lf %s",
"GPRINT:pcap2:AVERAGE:Average capt2 %9.0lf %s", 
"GPRINT:pcap2:LAST:Current capt2 %9.0lf %s\\r", 

"GPRINT:pdrop2:MAX:Max drop2 %9.0lf %s",
"GPRINT:pdrop2:AVERAGE:Average drop2 %9.0lf %s", 
"GPRINT:pdrop2:LAST:Current drop2 %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to graph `$pkts': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $pkts: $ERROR\n";
#	print "$time, error with rrd/update on $pkts: $ERROR\n";
};

}

# update and make png file for fixed ipproto
#####################################################

sub ipproto_update
{

RRDs::update $ipproto, "$t:"."$_[2]".":"."$_[4]".":"."$_[6]".":"."$_[8]";
 if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$ipproto': $ERROR\n";
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error with rrd/update on $ipproto: $ERROR\n";
  };

graph_ipproto("ipproto.daily-png", "daily: Fixed IP Protocols"." : ".$start_time, "-2000m");
graph_ipproto("ipproto.weekly-png", "weekly: Fixed IP Protocols"." : ".$start_time, "-12000m");
graph_ipproto("ipproto.monthly-png", "monthly: Fixed IP Protocols"." : ".$start_time, "-800h");
graph_ipproto("ipproto.yearly-png", "yearly: Fixed IP Protocols"." : ".$start_time, "-400d");

#RRDs::graph "ipproto.png",
#  "--title", "Fixed IP Protocols",
#  "--vertical-label", "bytes",
#  "--imgformat","PNG",
#  "DEF:a=$ipproto:tcp:AVERAGE",
#  "DEF:b=$ipproto:udp:AVERAGE",
#  "DEF:c=$ipproto:icmp:AVERAGE",
#  "DEF:d=$ipproto:xtra:AVERAGE",
#  "LINE1:a#0000ff:$_[1]",
#  "LINE1:b#00ff00:$_[3]",
#  "LINE1:c#ff0000:$_[5]",
#  "LINE1:d#0ffff0:$_[7]",
#;

}

# stored as bytes, multiple by 8 here and divide by 30 (secs)
# to get bits per second
#
sub graph_ipproto
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "bits/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
"DEF:tcp=$ipproto:tcp:AVERAGE",
"CDEF:ptcp=tcp,8,*,30,/",

"DEF:udp=$ipproto:udp:AVERAGE",
"CDEF:pudp=udp,8,*,30,/",

"DEF:icmp=$ipproto:icmp:AVERAGE",
"CDEF:picmp=icmp,8,*,30,/",

"DEF:xtra=$ipproto:xtra:AVERAGE",
"CDEF:pxtra=xtra,8,*,30,/",

"LINE1:ptcp#0000ff:tcp",
"LINE1:pudp#00ff00:udp",
"LINE1:picmp#ff0000:icmp",
"LINE1:pxtra#ff00ff:xtra",

#"COMMENT:               \\r",
"COMMENT:\\s",
"COMMENT:\\s",
"GPRINT:ptcp:MAX:Max tcp %9.0lf %s",
"GPRINT:ptcp:AVERAGE:Average tcp %9.0lf %s", 
"GPRINT:ptcp:LAST:Current tcp %9.0lf %s\\r", 

"GPRINT:pudp:MAX:Max udp %9.0lf %s",
"GPRINT:pudp:AVERAGE:Average udp %9.0lf %s", 
"GPRINT:pudp:LAST:Current udp %9.0lf %s\\r", 

"GPRINT:picmp:MAX:Max icmp %9.0lf %s",
"GPRINT:picmp:AVERAGE:Average icmp %9.0lf %s", 
"GPRINT:picmp:LAST:Current icmp %9.0lf %s\\r", 

"GPRINT:pxtra:MAX:Max xtra %9.0lf %s",
"GPRINT:pxtra:AVERAGE:Average xtra %9.0lf %s", 
"GPRINT:pxtra:LAST:Current xtra %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$ipproto': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $ipproto: $ERROR\n";
#	print "$time, error with rrd/update on $ipproto: $ERROR\n";
};

}

# update and make png file for fixed tcp3
#####################################################

sub tcp3_update
{

RRDs::update $tcp3, "$t:"."$_[2]".":"."$_[4]".":"."$_[6]";

if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$tcp3': $ERROR\n";
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    Update ERROR : $ERROR\n" if $ERROR;
};

graph_tcp3("tcp3.daily-png", "daily: 2 Fixed TCP dst ports"." : ".$start_time, "-2000m",
	$_[1], $_[3], $_[5]);
graph_tcp3("tcp3.weekly-png", "weekly: 2 Fixed TCP dst ports"." : ".$start_time, "-12000m",
	$_[1], $_[3], $_[5]);
graph_tcp3("tcp3.monthly-png", "monthly: 2 Fixed TCP dst ports"." : ".$start_time, "-800h",
	$_[1], $_[3], $_[5]);
graph_tcp3("tcp3.yearly-png", "yearly: 2 Fixed TCP dst ports"." : ".$start_time, "-400d",
	$_[1], $_[3], $_[5]);

#RRDs::graph "tcp3.png",
#  "--title", "Fixed TCP Ports",
#  "--vertical-label", "bytes",
#  "--imgformat","PNG",
#  "DEF:port1=$tcp3:port1:AVERAGE",
#  "DEF:port2=$tcp3:port2:AVERAGE",
#  "DEF:xtra=$tcp3:xtra:AVERAGE",
#  "LINE1:port1#0000ff:$_[1]",
#  "LINE1:port2#00ff00:$_[3]",
#  "LINE1:xtra#ff0000:$_[5]",
#;

}

sub graph_tcp3
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
my $label1 = $_[3];
my $label2 = $_[4];
my $label3 = $_[5];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "bits/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
"DEF:port1=$tcp3:port1:AVERAGE",
"CDEF:pport1=port1,8,*,30,/",

"DEF:port2=$tcp3:port2:AVERAGE",
"CDEF:pport2=port2,8,*,30,/",

"DEF:xtra=$tcp3:xtra:AVERAGE",
"CDEF:pxtra=xtra,8,*,30,/",


#  "LINE1:port1#0000ff:$_[1]",
#  "LINE1:port2#00ff00:$_[3]",
#  "LINE1:xtra#ff0000:$_[5]",

"LINE1:pport1#0000ff:$label1",
"LINE1:pport2#00ff00:$label2",
"LINE1:pxtra#ff0000:$label3",

"COMMENT:\\s",
"COMMENT:\\s",

"GPRINT:pport1:MAX:Max port1 %9.0lf %s",
"GPRINT:pport1:AVERAGE:Average port1 %9.0lf %s", 
"GPRINT:pport1:LAST:Current port1 %9.0lf %s\\r", 

"GPRINT:pport2:MAX:Max port2 %9.0lf %s",
"GPRINT:pport2:AVERAGE:Average port2 %9.0lf %s", 
"GPRINT:pport2:LAST:Current port2 %9.0lf %s\\r", 

"GPRINT:pxtra:MAX:Max xtra %9.0lf %s",
"GPRINT:pxtra:AVERAGE:Average xtra %9.0lf %s", 
"GPRINT:pxtra:LAST:Current xtra %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$tcp3': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $tcp3: $ERROR\n";
#	print "$time, error with rrd/update on $tcp3: $ERROR\n";
};

}

# update and make png file for iprange
##############################################################

sub iprange1_update
{
RRDs::update $iprange1, "$t:"."$_[2]".":"."$_[4]";
if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$iprange1': $ERROR\n";
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    unable to rrd/update $iprange1: $ERROR\n";
};

graph_iprange1("iprange1.daily-png", "daily: Fixed IP range"." : ".$start_time, "-2000m",
	$_[1]);
graph_iprange1("iprange1.weekly-png", "weekly: 2 Fixed IP range"." : ".$start_time, "-12000m",
	$_[1]);
graph_iprange1("iprange1.monthly-png", "monthly: 2 Fixed IP range"." : ".$start_time, "-800h",
	$_[1]);
graph_iprange1("iprange1.yearly-png", "yearly: 2 Fixed IP range"." : ".$start_time, "-400d",
	$_[1]);

# note:
# string labels are supplied from mon.lite file 
#RRDs::graph "iprange1.png",
#  "--title", "Fixed IP Range",
#  "--vertical-label", "bytes",
#  "--imgformat","PNG",
#  "DEF:range=$iprange1:range:AVERAGE",
#  "DEF:xtra=$iprange1:xtra:AVERAGE",
#  "LINE1:range#0000ff:$_[1]",
#  "LINE1:xtra#00ff00:$_[3]",
#;
}

sub graph_iprange1
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
my $label1 = $_[3];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "bits/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
"DEF:extra=$iprange1:xtra:AVERAGE",
"CDEF:pxtra=extra,8,*,30,/",
"DEF:range=$iprange1:range:AVERAGE",
"CDEF:prange=range,8,*,30,/",
"LINE1:prange#0000ff:$label1",
"LINE1:pxtra#ff0000:xtra",

"COMMENT:\\s",
"COMMENT:\\s",

"GPRINT:prange:MAX:Max range %9.0lf %s",
"GPRINT:prange:AVERAGE:Average range %9.0lf %s", 
"GPRINT:prange:LAST:Current range %9.0lf %s\\r", 

"GPRINT:pxtra:MAX:Max xtra %9.0lf %s",
"GPRINT:pxtra:AVERAGE:Average xtra %9.0lf %s", 
"GPRINT:pxtra:LAST:Current xtra %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $iprange1: $ERROR\n";
};

}


##############################################################
#
#
#
# update and make png file for cast filter
#
#
#
##############################################################

sub cast_update
{
RRDs::update $cast, "$t:"."$_[2]".":"."$_[4]".":"."$_[6]".":"."$_[8]";
if ($ERROR = RRDs::error) {
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error on rrd/update for $cast: $ERROR\n";
};

graph_cast("cast.daily-png", "daily: Fixed M/U/B cast pkts"." : ".$start_time, "-2000m",
	$_[1], $_[3], $_[5], $_[7]);
graph_cast("cast.weekly-png", "weekly: Fixed M/U/B cast pkts"." : ".$start_time, "-12000m",
	$_[1], $_[3], $_[5], $_[7]);
graph_cast("cast.monthly-png", "monthly: Fixed M/U/B cast pkts"." : ".$start_time, "-800h",
	$_[1], $_[3], $_[5], $_[7]);
graph_cast("cast.yearly-png", "yearly: Fixed M/U/B cast pkts"." : ".$start_time, "-400d",
	$_[1], $_[3], $_[5], $_[7]);
 
# note
# string labels are supplied from mon.lite file 
#RRDs::graph "cast.png",
#  "--title", "mcast vs. ucast vs. bcast vs. xtra",
#  "--vertical-label", "bytes",
#  "--imgformat","PNG",
#  "DEF:mcast=$cast:mcast:AVERAGE",
#  "DEF:ucast=$cast:ucast:AVERAGE",
#  "DEF:bcast=$cast:bcast:AVERAGE",
#  "DEF:xtra=$cast:xtra:AVERAGE",
#  "LINE1:mcast#0000ff:$_[1]",
#  "LINE1:ucast#00ff00:$_[3]",
#  "LINE1:bcast#ff0000:$_[5]",
#  "LINE1:xtra#0ffff0:$_[7]",
#;
}

sub graph_cast
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
my $label1 = $_[3];
my $label2 = $_[4];
my $label3 = $_[5];
my $label4 = $_[6];

#print $label1, $label2, $label3, $label4, "\n";
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "bits/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
#  "DEF:ucast=$cast:ucast:AVERAGE",
#  "DEF:bcast=$cast:bcast:AVERAGE",
#  "DEF:xtra=$cast:xtra:AVERAGE",

"DEF:mcast=$cast:mcast:AVERAGE",
"CDEF:pmcast=mcast,8,*,30,/",

"DEF:ucast=$cast:ucast:AVERAGE",
"CDEF:pucast=ucast,8,*,30,/",

"DEF:bcast=$cast:bcast:AVERAGE",
"CDEF:pbcast=bcast,8,*,30,/",

"DEF:xtra=$cast:xtra:AVERAGE",
"CDEF:pxtra=xtra,8,*,30,/",

"LINE1:pmcast#0000ff:$label1",
"LINE1:pucast#00ff00:$label2",
"LINE1:pbcast#ff0000:$label3",
"LINE1:pxtra#ff00ff:$label4",

"COMMENT:\\s",
"COMMENT:\\s",

"GPRINT:pmcast:MAX:Max mcast %9.0lf %s",
"GPRINT:pmcast:AVERAGE:Average mcast %9.0lf %s", 
"GPRINT:pmcast:LAST:Current mcast %9.0lf %s\\r", 

"GPRINT:pucast:MAX:Max ucast %9.0lf %s",
"GPRINT:pucast:AVERAGE:Average ucast %9.0lf %s", 
"GPRINT:pucast:LAST:Current ucast %9.0lf %s\\r", 

"GPRINT:pbcast:MAX:Max bcast %9.0lf %s",
"GPRINT:pbcast:AVERAGE:Average bcast %9.0lf %s", 
"GPRINT:pbcast:LAST:Current bcast %9.0lf %s\\r", 

"GPRINT:pxtra:MAX:Max xtra %9.0lf %s",
"GPRINT:pxtra:AVERAGE:Average xtra %9.0lf %s", 
"GPRINT:pxtra:LAST:Current xtra %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $cast: $ERROR\n";
} 

}
  


#
# create the RRD file for the pkts filter
#
sub pkts_create
{

########################################################################
#
#  $start-1  
#                the time slot for starting 
#  "--step", 30
#                data updated for every 30 seconds 
#  DS:a, DS:b    (count/dropped for if 1, and same for i/f 2)
#                2 data sources, all is GAUGE type
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#
# mrtg samples
#        600 5-minute samples:    2   days and 2 hours
#        600 30-minute samples:  12.5 days
#        600 2-hour samples:     50   days
#        600 1-day samples:     732   days
# therefore we take the mrtg scheme which is:
#                "RRA:AVERAGE:0.5:1:600",   
#                "RRA:AVERAGE:0.5:6:700",    
#                "RRA:AVERAGE:0.5:24:775", 
#                "RRA:AVERAGE:0.5:288:797",
#                "RRA:MAX:0.5:1:600",   
#                "RRA:MAX:0.5:6:700",     
#                "RRA:MAX:0.5:24:775",  
#                "RRA:MAX:0.5:288:797");
# and simply update it for 30 second sample periods,
# as opposed to 300 second sample periods (* 10)
#
# 1 sample per 30 seconds for 2 days plus,
# everything else * 10 for number of samples, 30 min. 2 hrs, 1 day

RRDs::create ($pkts, "--start",$start-1, "--step",30,
	"DS:cap1:GAUGE:600:U:U",
        "DS:drop1:GAUGE:600:U:U",
        "DS:cap2:GAUGE:600:U:U",
        "DS:drop2:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);

my $epkts = RRDs::error;
print ERR "rrd creation ERROR : $epkts\n" if $epkts;

}

sub dnsstats_create
{

########################################################################
#
#  $start-1  
#                the time slot for starting 
#  "--step", 30
#                data updated for every 30 seconds 
#  DS:a, DS:b    (count/dropped for if 1, and same for i/f 2)
#                2 data sources, all is GAUGE type
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#
# mrtg samples
#        600 5-minute samples:    2   days and 2 hours
#        600 30-minute samples:  12.5 days
#        600 2-hour samples:     50   days
#        600 1-day samples:     732   days
# therefore we take the mrtg scheme which is:
#                "RRA:AVERAGE:0.5:1:600",   
#                "RRA:AVERAGE:0.5:6:700",    
#                "RRA:AVERAGE:0.5:24:775", 
#                "RRA:AVERAGE:0.5:288:797",
#                "RRA:MAX:0.5:1:600",   
#                "RRA:MAX:0.5:6:700",     
#                "RRA:MAX:0.5:24:775",  
#                "RRA:MAX:0.5:288:797");
# and simply update it for 30 second sample periods,
# as opposed to 300 second sample periods (* 10)
#
# 1 sample per 30 seconds for 2 days plus,
# everything else * 10 for number of samples, 30 min. 2 hrs, 1 day

RRDs::create ($dnsstats, "--start",$start-1, "--step",30,
	"DS:qy:GAUGE:600:U:U",
        "DS:qr:GAUGE:600:U:U",
        "DS:arec:GAUGE:600:U:U",
        "DS:a4rec:GAUGE:600:U:U",
        "DS:mxrec:GAUGE:600:U:U",
        "DS:ptrrec:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);

my $epkts = RRDs::error;
print ERR "rrd creation ERROR : $epkts\n" if $epkts;

}

####################################################################
#    update and make png file for dnsstats capture
#################################################################
sub dnsstats_update 
{ 
my $graph = "dnsstats";

# do update
RRDs::update $dnsstats, "$t:"."$_[1]".":"."$_[2]".":"."$_[3]".":"."$_[4]"
.":"."$_[5]".":"."$_[6]";
if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$dnsstats': $ERROR\n";
      $time=`date`;
      $time=~ s/\n//g;
      print ERR "$time, error with rrd/update on $dnsstats: $ERROR\n";
};

# args are 1. filename,  2. title,  3. -s time back from now
#
graph_dns("$graph.daily-png", "daily: dns stats"." : ".$start_time, "-2000m");
graph_dns("$graph.weekly-png", "weekly: dns stats"." : ".$start_time, "-12000m");
graph_dns("$graph.monthly-png", "monthly: dns stats"." : ".$start_time, "-800h");
graph_dns("$graph.yearly-png", "yearly: dns stats"." : ".$start_time, "-400d");

}

# spit out rrdtool-speak to graph the PKTS 4 graphs

sub graph_dns
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "pkts/sec",
"-w", "500",
"-h",  "100", 
"-b", "1024", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
"DEF:qy=$dnsstats:qy:AVERAGE",
"CDEF:pqy=qy,30,/",
"DEF:qr=$dnsstats:qr:AVERAGE",
"CDEF:pqr=qr,30,/",
"DEF:arec=$dnsstats:arec:AVERAGE",
"CDEF:parec=arec,30,/",
"DEF:a4rec=$dnsstats:a4rec:AVERAGE",
"CDEF:pa4rec=a4rec,30,/",
"DEF:mxrec=$dnsstats:mxrec:AVERAGE",
"CDEF:pmxrec=mxrec,30,/",
"DEF:ptrrec=$dnsstats:ptrrec:AVERAGE",
"CDEF:pptrrec=ptrrec,30,/",

"LINE1:pqy#0000ff:qy",
"LINE1:pqr#00ff00:qr",
"LINE1:parec#ff0000:A",
"LINE1:pa4rec#ff00ff:AAAA",
"LINE1:pmxrec#00ffff:MX",
"LINE1:pptrrec#ffff00:PTR",
#"COMMENT:               \\r",
"COMMENT:\\s",
"COMMENT:\\s",
"GPRINT:pqy:MAX:Max qy %9.0lf %s",
"GPRINT:pqy:AVERAGE:Average qy %9.0lf %s", 
"GPRINT:pqy:LAST:Current qy %9.0lf %s\\r", 

"GPRINT:pqr:MAX:Max qr %9.0lf %s",
"GPRINT:pqr:AVERAGE:Average qr %9.0lf %s", 
"GPRINT:pqr:LAST:Current qr %9.0lf %s\\r", 

"GPRINT:parec:MAX:Max A %9.0lf %s",
"GPRINT:parec:AVERAGE:Average A %9.0lf %s", 
"GPRINT:parec:LAST:Current A %9.0lf %s\\r", 

"GPRINT:pa4rec:MAX:Max AAAA %9.0lf %s",
"GPRINT:pa4rec:AVERAGE:Average AAAA %9.0lf %s", 
"GPRINT:pa4rec:LAST:Current AAAA %9.0lf %s\\r", 

"GPRINT:pmxrec:MAX:Max MX %9.0lf %s",
"GPRINT:pmxrec:AVERAGE:Average MX %9.0lf %s", 
"GPRINT:pmxrec:LAST:Current MX %9.0lf %s\\r", 

"GPRINT:pptrrec:MAX:Max PTR %9.0lf %s",
"GPRINT:pptrrec:AVERAGE:Average PTR %9.0lf %s", 
"GPRINT:pptrrec:LAST:Current PTR %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to graph `$dnsstats': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $dnsstats: $ERROR\n";
};

}

sub ipproto_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                 zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#
########################################################################

RRDs::create ($ipproto, "--start",$start-1, "--step", 30,
	"DS:tcp:GAUGE:600:U:U",
        "DS:udp:GAUGE:600:U:U",
        "DS:icmp:GAUGE:600:U:U",
        "DS:xtra:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);

my $eipproto = RRDs::error;
print ERR "rrd creation ERROR : $eipproto\n" if $eipproto;
}

sub tcp3_create
{
######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c
#                3 data sources, all is GAUGE type, since theay are already
#                zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#
########################################################################

RRDs::create ($tcp3, "--start",$start-1, "--step", 30,
	"DS:port1:GAUGE:600:U:U",
        "DS:port2:GAUGE:600:U:U",
        "DS:xtra:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);

my $etcp3 = RRDs::error;
print ERR "rrd creation ERROR : $etcp3\n" if $etcp3;
}

sub iprange1_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b 
#                2 data sources, all is GAUGE type, since theay are already
#                zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#
########################################################################

RRDs::create ($iprange1, "--start",$start-1, "--step",30,
	"DS:range:GAUGE:600:U:U",
        "DS:xtra:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);
my $eiprange1 = RRDs::error;
print ERR "rrd creation ERROR : $eiprange1\n" if $eiprange1;
}

sub cast_create
{
########################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
# 
########################################################################

RRDs::create ($cast, "--start",$start-1, "--step",30,
        "DS:mcast:GAUGE:600:U:U",
        "DS:ucast:GAUGE:600:U:U",
        "DS:bcast:GAUGE:600:U:U",
        "DS:xtra:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);
my $ecast = RRDs::error;
print ERR "rrd creation ERROR : $ecast\n" if $ecast;
}
 
sub size_create
{
########################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
# 
########################################################################

RRDs::create ($size, "--start",$start-1, "--step",30,
        "DS:tiny:GAUGE:600:U:U",
        "DS:small:GAUGE:600:U:U",
        "DS:med:GAUGE:600:U:U",
        "DS:large:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);
my $esize = RRDs::error;
print ERR "rrd creation ERROR : $esize\n" if $esize;
}


sub size_update
{
RRDs::update $size, "$t:"."$_[2]".":"."$_[4]".":"."$_[6]".":"."$_[8]";
if ($ERROR = RRDs::error) {
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error on rrd/update for $size: $ERROR\n";
};

graph_size("size.daily-png", "daily: packet sizes (100/500/1000/1500)", "-2000m",
	$_[1], $_[3], $_[5], $_[7]);
graph_size("size.weekly-png", "weekly: packet sizes (100/500/1000/1500)", "-12000m",
	$_[1], $_[3], $_[5], $_[7]);
graph_size("size.monthly-png", "monthly: packet sizes (100/500/1000/1500)", "-800h",
	$_[1], $_[3], $_[5], $_[7]);
graph_size("size.yearly-png", "yearly: packet sizes (100/500/1000/1500)", "-400d",
	$_[1], $_[3], $_[5], $_[7]);
}

sub graph_size
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
my $label1 = $_[3];
my $label2 = $_[4];
my $label3 = $_[5];
my $label4 = $_[6];

#print $label1, $label2, $label3, $label4, "\n";
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "pkts/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 

"DEF:tiny=$size:tiny:AVERAGE",
"CDEF:ptiny=tiny,30,/",
#"CDEF:ptiny=tiny,8,*,30,/",

"DEF:small=$size:small:AVERAGE",
"CDEF:psmall=small,30,/",
#"CDEF:psmall=small,8,*,30,/",

"DEF:med=$size:med:AVERAGE",
"CDEF:pmed=med,30,/",
#"CDEF:pmed=med,8,*,30,/",

"DEF:large=$size:large:AVERAGE",
"CDEF:plarge=large,30,/",
#"CDEF:plarge=large,8,*,30,/",

"LINE1:ptiny#0000ff:$label1",
"LINE1:psmall#00ff00:$label2",
"LINE1:pmed#ff0000:$label3",
"LINE1:plarge#ff00ff:$label4",

"COMMENT:\\s",
"COMMENT:\\s",

"GPRINT:ptiny:MAX:Max tiny %9.0lf %s",
"GPRINT:ptiny:AVERAGE:Average tiny %9.0lf %s", 
"GPRINT:ptiny:LAST:Current tiny %9.0lf %s\\r", 

"GPRINT:psmall:MAX:Max small %9.0lf %s",
"GPRINT:psmall:AVERAGE:Average small %9.0lf %s", 
"GPRINT:psmall:LAST:Current small %9.0lf %s\\r", 

"GPRINT:pmed:MAX:Max med %9.0lf %s",
"GPRINT:pmed:AVERAGE:Average med %9.0lf %s", 
"GPRINT:pmed:LAST:Current med %9.0lf %s\\r", 

"GPRINT:plarge:MAX:Max large %9.0lf %s",
"GPRINT:plarge:AVERAGE:Average large %9.0lf %s", 
"GPRINT:plarge:LAST:Current large %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $size: $ERROR\n";
} 

}


@glabels;	# global list of bpf labels (individual graph labels)
@gcounts;	# global list of per label counts 
@glen;		# count of arguments

# boop
# syntax: bpf tag #pairs label count [label count]* ...  xtra count
sub bpf_update
{
	$line = $_[0];
	my $tag;
	my $uberlabel;
	my @words;
	my $index;
	my $labelpairs;
	my $rrdname;
	my $xtracount;
	my @cargs;
	my $countpkts;		# bytes or packets for label 

     	@words = split(/:/,$line);
	$tag = $uberlabel = $words[1];	# this is the graph label with blanks 
     	$tag =~ s/ //g;    # without blanks
	$countpkts = $words[2];	# flag: packets or bytes for count
	$labelpairs = $words[3];	# count of bpfs

	# put per graph labels in an array
	$index = 4;
	for ( $i = 0; $i < $labelpairs; $i++) {
		$glabels[$i] = $words[$index];
		$gcounts[$i] = $words[$index+1];
		$index = $index + 2;
	}
	$index++;
	$xtracount = $words[$index]; 
	$glen = $labelpairs;

	# take label and eliminate blanks.
	# then create rrd file if necessary
	#
	$rrdname = $rrdbase . "\/" . "$tag.rrd";
	unless (open(HANDLE, $rrdname)) {
		bpf_create($rrdname, @glabels);
	};	
	close(HANDLE);

	# update rrd
	$cargs[0] = $rrdname;
	$cargs[1] = "$t:";
	for ( $i = 0; $i < $labelpairs; $i++) {
		$cargs[1] = $cargs[1] . "$gcounts[$i]:"; 
	}
	$cargs[1] = $cargs[1] . "$xtracount"; 

	RRDs::update(@cargs);
 	if ($ERROR = RRDs::error) {
		#    die "$0: unable to update `$rrdname': $ERROR\n";
     		$time=`date`;
     		$time=~ s/\n//g;
     		print ERR "$time    error with rrd/update on $rrdname: $ERROR\n";
  	};

	# graph pictures
	graph_bpf($rrdname, "bpf.$tag.daily-png", 
		"daily: $uberlabel"." : ".$start_time, "-2000m", 
		$labelpairs, $xtracount, $countpkts);
	graph_bpf($rrdname, "bpf.$tag.weekly-png", 
		"weekly: $uberlabel"." : ".$start_time, "-12000m", 
		$labelpairs, $xtracount, $countpkts);
	graph_bpf($rrdname, "bpf.$tag.monthly-png", 
		"monthly: $uberlabel "." : ".$start_time, "-800h", 
		$labelpairs, $xtracount, $countpkts);
	graph_bpf($rrdname, "bpf.$tag.yearly-png", 
		"yearly: $uberlabel "." : ".$start_time, "-400d", 
		$labelpairs, $xtracount, $countpkts);

}

sub graph_bpf
{
	my $rrdname = $_[0];
	my $fname = $_[1];
	my $title = $_[2];
	my $stime = $_[3];
	my $pairs = $_[4];
	my $xtracount = $_[5];
	my $countpkts = $_[6];
	my @cargs;
	my $cargsindex;
	my $i;
	my $rgb;

	$cargs[0] = $fname;
	$cargs[1] = "--title"; 
	$cargs[2] = $title;
	$cargs[3] = "-s"; 		# start
	$cargs[4] = $stime;		# start seconds
	$cargs[5] = "-e"; 		# end
	$cargs[6] = "now";		# end time 
	$cargs[7] = "-a"; 		# image format
	$cargs[8] = "PNG";		# which is PNG
	$cargs[9] = "-v"; 		# vertical label
	$cargs[10] = "bits/sec";	# arg to vertical label
	if ($countpkts) {
		$cargs[10] = "pkts/sec";
	}
	$cargs[11] = "-b"; $cargs[12] = "1000";		# base is 1000
	$cargs[13] = "--alt-y-grid"; 
	#$cargs[13] = "-A";
	$cargs[14] = "--lazy";
	$cargs[15] = "-c";  $cargs[16] = "MGRID#ee0000";
	$cargs[17] = "-c";  $cargs[18] = "GRID#000000";

	$cargsindex = 19;
	for ($i = 0; $i < $pairs; $i++) {
		$cargs[$cargsindex++] = "DEF:$glabels[$i]=$rrdname:$glabels[$i]:AVERAGE";
		# for packets divide by 30 (pkts/sec) 
		# for bytes multiply by 8 and divide by 30 (bits/sec)
		if ($countpkts) {
		 	$cargs[$cargsindex++] = "CDEF:p$glabels[$i]=$glabels[$i],30,/";
		}
		else {
			$cargs[$cargsindex++] = "CDEF:p$glabels[$i]=$glabels[$i],8,*,30,/";
		}
	}
	$cargs[$cargsindex++] = "DEF:xtra=$rrdname:xtra:AVERAGE";
	if ($countpkts) {
		$cargs[$cargsindex++] = "CDEF:pxtra=xtra,30,/";
	}
	else {
		$cargs[$cargsindex++] = "CDEF:pxtra=xtra,8,*,30,/";
	}

	for ($i = 0; $i < $pairs; $i++) {
		$rgb = getrgb($i);
		$cargs[$cargsindex++] = "LINE1:p$glabels[$i]#$rgb:$glabels[$i]";
	}
	$rgb = getrgb($i);
	$cargs[$cargsindex++] = "LINE2:pxtra#$rgb:xtra";

	$cargs[$cargsindex++] = "COMMENT:\\s";
	$cargs[$cargsindex++] = "COMMENT:\\s";

	for ($i = 0; $i < $pairs; $i++) {
		$cargs[$cargsindex++] = "GPRINT:p$glabels[$i]:MAX:Max $glabels[$i] %8.0lf %s";
		$cargs[$cargsindex++] = "GPRINT:p$glabels[$i]:AVERAGE:Avg $glabels[$i] %8.0lf %s";
		$cargs[$cargsindex++] = "GPRINT:p$glabels[$i]:LAST:Cur $glabels[$i] %8.0lf %s\\s";
		$cargs[$cargsindex++] = "COMMENT:\\s";
	}
	$cargs[$cargsindex++] = "GPRINT:pxtra:MAX:Max xtra %8.0lf %s";
	$cargs[$cargsindex++] = "GPRINT:pxtra:AVERAGE:Avg xtra %8.0lf %s";
	$cargs[$cargsindex++] = "GPRINT:pxtra:LAST:Cur xtra %8.0lf %s";
 
	RRDs::graph( @cargs );

	if ($ERROR = RRDs::error) {
	#    die "$0: unable to update `$ipproto': $ERROR\n";
      		$time=`date`;
      		$time=~ s/\n//g;
 		print ERR "$time, error with rrd/graph on $rrdname: $ERROR\n";
	#	print "$time, error with rrd/update on $rrdname: $ERROR\n";
	};
}

# bpf_create($filename, @labels)
#
# create bpf rrd in input filename, with variable number of DS entries
sub bpf_create
{
	my @labels = @_;
	my $name = $labels[0];	# name of rrd file
#	my $labelcount = $#labels;
 	my $labelcount = $glen;
	my $i;
	my @cargs;
	my $cargsindex = 5;

	$cargs[0] = $name; 
	$cargs[1] = "--start";
	$cargs[2] = $start-1; 
	$cargs[3] = "--step"; 
	$cargs[4] = 30;
	for ($i = 1; $i < ($labelcount + 1); $i++) {
		$cargs[$cargsindex++] = "DS:$labels[$i]:GAUGE:600:U:U",
	}
	$cargs[$cargsindex++] = "DS:xtra:GAUGE:600:U:U",
	$cargs[$cargsindex++] = "RRA:AVERAGE:0.5:1:6000";
	$cargs[$cargsindex++] = "RRA:AVERAGE:0.5:60:700";
	$cargs[$cargsindex++] = "RRA:AVERAGE:0.5:240:775";
	$cargs[$cargsindex++] = "RRA:AVERAGE:0.5:2880:797";
	$cargs[$cargsindex++] = "RRA:MAX:0.5:1:6000";
	$cargs[$cargsindex++] = "RRA:MAX:0.5:60:700";
	$cargs[$cargsindex++] = "RRA:MAX:0.5:240:775";
	$cargs[$cargsindex++] = "RRA:MAX:0.5:2880:797";

	RRDs::create (@cargs);

	my $eipproto = RRDs::error;
	print ERR "rrd creation ERROR : $eipproto\n" if $eipproto;
}

# make colors for labels ... this routine needs work
#
sub getrgb
{
	my $index = $_[0];
	my $n;

	#print $_0, " ", "$index\n";
	$n = $index % 6;
	if ($n eq 0) {
		return("0000ff");
	}
	elsif ($n eq 1) {
		return("00ff00");
	}
	elsif ($n eq 2) {
		return("ff0000");
	}
	elsif ($n eq 3) {
		#return("00fff0");
		# purple  red/blue
		return("ff00ff");
	}
	elsif ($n eq 4) {
		# yellow
		return("ffff00");
	}
	elsif ($n eq 5) {
		# cyan	
		return("00ffff");
		
	}
	return("f0f0f0");
}

#
# put the alert in the alert log file
# input params: flow plus bit count
# make a line, and quote for parens because they make
#       the shell unhappy.   
sub alert_log
{
	my $flow = $_[0];
	my $count = $_[1];
         
  	# ignore xtra as it is the remainder for top_ip
  	if ($flow eq "xtra") {
        	return;
  	}
  	$logline = $flow . " " . $count;
  	if ($logging) {
		omsyslog($logbase, "alert.log", $logline);
  	}
}


# ------------------------------------------------------------>
# update and make png file for flowcount
#####################################################

sub flowcount_update
{

unless (open(HANDLE, $flowcount)) {
	flowcount_create();
};	
close(HANDLE);

RRDs::update $flowcount, "$_[0]";
 if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$flowcount': $ERROR\n";
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error with rrd/update on $flowcount: $ERROR\n";
  };

graph_flowcount("flowcount.daily-png", "daily: FLOWS "." : ".$start_time, "-2000m");
graph_flowcount("flowcount.weekly-png", "weekly: FLOWS "." : ".$start_time, "-12000m");
graph_flowcount("flowcount.monthly-png", "monthly: FLOWS "." : ".$start_time, "-800h");
graph_flowcount("flowcount.yearly-png", "yearly: FLOWS "." : ".$start_time, "-400d");

#RRDs::graph "flowcount.png",
#  "--title", "FLOWS",
#  "--vertical-label", "bytes",
#  "--imgformat","PNG",
#  "DEF:a=$flowcount:tcp:AVERAGE",
#  "DEF:b=$flowcount:udp:AVERAGE",
#  "DEF:c=$flowcount:icmp:AVERAGE",
#  "DEF:d=$flowcount:xtra:AVERAGE",
#  "LINE1:a#0000ff:$_[1]",
#  "LINE1:b#00ff00:$_[3]",
#  "LINE1:c#ff0000:$_[5]",
#  "LINE1:d#0ffff0:$_[7]",
#;

}

# stored as flows, divide by 30 (secs)
#
sub graph_flowcount
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "flows/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 

"DEF:ip=$flowcount:ip:AVERAGE",
"CDEF:pip=ip,30,/",

"DEF:tcp=$flowcount:tcp:AVERAGE",
"CDEF:ptcp=tcp,30,/",

"DEF:udp=$flowcount:udp:AVERAGE",
"CDEF:pudp=udp,30,/",

"DEF:icmp=$flowcount:icmp:AVERAGE",
"CDEF:picmp=icmp,30,/",


"LINE1:pip#0000ff:ip",
"LINE1:ptcp#00ff00:tcp",
"LINE1:pudp#ff0000:udp",
"LINE1:picmp#ff00ff:icmp",

#"COMMENT:               \\r",
"COMMENT:\\s",
"COMMENT:\\s",
"GPRINT:pip:MAX:Max ip %9.0lf %s",
"GPRINT:pip:AVERAGE:Average ip %9.0lf %s", 
"GPRINT:pip:LAST:Current ip %9.0lf %s\\r", 

"GPRINT:ptcp:MAX:Max tcp %9.0lf %s",
"GPRINT:ptcp:AVERAGE:Average tcp %9.0lf %s", 
"GPRINT:ptcp:LAST:Current tcp %9.0lf %s\\r", 

"GPRINT:pudp:MAX:Max udp %9.0lf %s",
"GPRINT:pudp:AVERAGE:Average udp %9.0lf %s", 
"GPRINT:pudp:LAST:Current udp %9.0lf %s\\r", 

"GPRINT:picmp:MAX:Max icmp %9.0lf %s",
"GPRINT:picmp:AVERAGE:Average icmp %9.0lf %s", 
"GPRINT:picmp:LAST:Current icmp %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$flowcount': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $flowcount: $ERROR\n";
#	print "$time, error with rrd/update on $flowcount: $ERROR\n";
};

}

sub flowcount_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                 zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#               
########################################################################
 
RRDs::create ($flowcount, "--start",$start-1, "--step", 30,
        "DS:ip:GAUGE:600:U:U",
        "DS:tcp:GAUGE:600:U:U",
        "DS:udp:GAUGE:600:U:U",
        "DS:icmp:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);

my $eflowcount = RRDs::error;
print ERR "rrd creation ERROR : $eflowcount\n" if $eflowcount;
}

#
# graph insert counter as it may provide clues
# about DOS attacks
# mon.lite:hs_stat: 42886: 656766: 357408: 247151: 52207: 31236: 52208: 11745:
#							         inserts
#
sub topn_stat
{
	my $insert = $_[7];

	unless (open(HANDLE, $topnstat)) {
		topnstat_create();
	};	
	close(HANDLE);


#	RRDs::update $topnstat, "$_[0]";
	RRDs::update $topnstat, "$t:"."$insert";
#print "$insert\n";
 	if ($ERROR = RRDs::error) {
     		$time=`date`;
     		$time=~ s/\n//g;
     		print ERR "$time    error with rrd/update on $topnstat: $ERROR\n";
  	};

	graph_topnstat("topnstat.daily-png", "daily: top inserts "." : ".$start_time, "-2000m");
	graph_topnstat("topnstat.weekly-png", "weekly: top inserts "." : ".$start_time, "-12000m");
	graph_topnstat("topnstat.monthly-png", "monthly: top inserts "." : ".$start_time, "-800h");
	graph_topnstat("topnstat.yearly-png", "yearly: top inserts "." : ".$start_time, "-400d");
}

# don't mod the insert number therefore per period
#
sub graph_topnstat
{

	my $fname = $_[0];
	my $title = $_[1];
	my $stime = $_[2];
 
	RRDs::graph( $fname,
	"--title", $title,
	"-s", $stime,
	"-e",  "now", 
	"-a",  "PNG", 
	"-v" , "count/period",
	"-b", "1000", 
	"--alt-y-grid", 
	"--lazy", 
	"-c", "MGRID#ee0000", 
	"-c", "GRID#000000", 

	"DEF:insert=$topnstat:insert:AVERAGE",
	"CDEF:pinsert=insert",
	"LINE1:pinsert#0000ff:insert",
	#"COMMENT:               \\r",
	"COMMENT:\\s",
	"COMMENT:\\s",
	"GPRINT:pinsert:MAX:Max insert %9.0lf %s",
	"GPRINT:pinsert:AVERAGE:Avg insert %9.0lf %s", 
	"GPRINT:pinsert:LAST:Last insert %9.0lf %s\\r", 
	);

	if ($ERROR = RRDs::error) {
		$time=`date`;
		$time=~ s/\n//g;
		print ERR "$time, error with rrd/graph on $topnsort: $ERROR\n";
	};

}

sub topnstat_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                 zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#               
########################################################################
 
	RRDs::create ($topnstat, "--start",$start-1, "--step", 30,
        	"DS:insert:GAUGE:600:U:U",

		"RRA:AVERAGE:0.5:1:6000",
		"RRA:AVERAGE:0.5:60:700",
		"RRA:AVERAGE:0.5:240:775",
		"RRA:AVERAGE:0.5:2880:797",

		"RRA:MAX:0.5:1:6000",
		"RRA:MAX:0.5:60:700",
		"RRA:MAX:0.5:240:775",
		"RRA:MAX:0.5:2880:797" );

	my $etopnstat = RRDs::error;
	print ERR "rrd creation ERROR : $etopnstat\n" if $etopnstat;
}

# graph tcp scan count with 3 measures
#	total: generated by this site: sent to this site
#
# This is a metadata graph that is generated as a side effect
# by the tcp topn syn counter.  Note: it is has nothing to do with
# any bpf tcp syn counter.
#
sub tworm_update
{
my $data;

unless (open(HANDLE, $twormcount)) {
	twormcount_create();
};	
close(HANDLE);

$data = "$t:" . $_[1] . ":" . $_[2] . ":" . $_[3];

RRDs::update $twormcount, $data;
 if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$twormcount': $ERROR\n";
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error with rrd/update on $twormcount: $ERROR\n";
  };

graph_twormcount("twormcount.daily-png", "daily: tcp scan count "." : ".$start_time, "-2000m");
graph_twormcount("twormcount.weekly-png", "weekly: tcp scan count "." : ".$start_time, "-12000m");
graph_twormcount("twormcount.monthly-png", "monthly: tcp scan count "." : ".$start_time, "-800h");
graph_twormcount("twormcount.yearly-png", "yearly: tcp scan count "." : ".$start_time, "-400d");

#RRDs::graph "twormcount.png",
#  "--title", "FLOWS",
#  "--vertical-label", "bytes",
#  "--imgformat","PNG",
#  "DEF:a=$twormcount:tcp:AVERAGE",
#  "DEF:b=$twormcount:udp:AVERAGE",
#  "DEF:c=$twormcount:icmp:AVERAGE",
#  "DEF:d=$twormcount:xtra:AVERAGE",
#  "LINE1:a#0000ff:$_[1]",
#  "LINE1:b#00ff00:$_[3]",
#  "LINE1:c#ff0000:$_[5]",
#  "LINE1:d#0ffff0:$_[7]",
#;

}

# do not divide by 30 seconds, we want this per period
#
# there are 3 lines: count, us, them.
sub graph_twormcount
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "count/period",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 

"DEF:count=$twormcount:count:AVERAGE",
"CDEF:pcount=count",

"DEF:us=$twormcount:us:AVERAGE",
"CDEF:pus=us",

"DEF:them=$twormcount:them:AVERAGE",
"CDEF:pthem=them",


"LINE1:pcount#0000ff:count",
"LINE2:pus#00ff00:us",
"LINE3:pthem#ff0000:them",

#"COMMENT:               \\r",
"COMMENT:\\s",
"COMMENT:\\s",
"GPRINT:pcount:MAX:Max count %9.0lf %s",
"GPRINT:pcount:AVERAGE:Average count %9.0lf %s", 
"GPRINT:pcount:LAST:Current count %9.0lf %s\\r", 

"GPRINT:pus:MAX:Max us %9.0lf %s",
"GPRINT:pus:AVERAGE:Average us %9.0lf %s", 
"GPRINT:pus:LAST:Current us %9.0lf %s\\r", 

"GPRINT:pthem:MAX:Max them %9.0lf %s",
"GPRINT:pthem:AVERAGE:Average them %9.0lf %s", 
"GPRINT:pthem:LAST:Current them %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$twormcount': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $twormcount: $ERROR\n";
#	print "$time, error with rrd/update on $twormcount: $ERROR\n";
};

}

sub twormcount_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                 zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#               
########################################################################
 
RRDs::create ($twormcount, "--start",$start-1, "--step", 30,
        "DS:count:GAUGE:600:U:U",
        "DS:us:GAUGE:600:U:U",
        "DS:them:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
	"RRA:AVERAGE:0.5:2880:797",

        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
	"RRA:MAX:0.5:2880:797" 
);

my $etwormcount = RRDs::error;
	print ERR "rrd creation ERROR : $etwormcount\n" if $etwormcount;
}

#
# note: for now we are going with a one-week backup
# scheme:
#	1. cron zeroes the next days dir e.g., logs/Mon
#	2. omupdate.pl creates the log file and appends to it.
#
# In the future, we might just create logs based on the Month/day
# e.g., date gives us Mon Apr 5, so we might go with a month_date scheme.
# This would give us a month at a time; e.g.,
#	1. at midnight, cron zeros out the current log directory, and
#	put in a mon_date stamp (Apr_5)
#	2. omupdate creates logs within that directory.
#
# more or less syslog format in terms
# of date (although the year is in the date)
# followed by message.
#
# note: we have the nice property that the file
# is created on 1st log entry.
#
# calling convention:
# $mylogdir="/tmp/foo.log";
# $mylogname="topn_tcp";
# omsyslog($mylogdirectory, $mylogname, "my message");
#
# globals: start_time
#
sub omsyslog
{
	my $logbase = $_[0];
	my $logfile = $_[1];
	my $msg = $_[2];
	my $local_start_time= $start_time;
	$local_start_time=~ s/  / /g;
	my @time = split(/ /, $local_start_time);
	my $syslog_time = $time[1] . ' ' . $time[2] . ' ' . $time[3] . ' ' . $time[4] . ' ' . $time[5];
	# pathname is log dir + day of week + logfile name.
	my $pathname = "$logbase/$time[0]/$logfile";
	
	open(LOG, ">>$pathname") || die "can't open $pathname: $!\n";
	print LOG "$syslog_time: $msg\n";
	close(LOG);
}


# storeEvent($eventfile, "new worm signature from:", $sbuf);				
#
# storeEvent(file, prependString, eventString)
#
# eventString may or may not have a newline.  we strip it if it does.
#
sub
storeEvent
{

	if ($eventflag == 0) {
		return;
	}
	my $eventfile = $_[0];
	my $prependString = $_[1];
	my $eventString = $_[2];
	my $s;

	$eventString=~ s/\n//g;
	$s = "$start_time: " . $prependString . $eventString;

 	open(FILE, ">>$eventfile") || die "can't open $eventfile: $!\n";
 	print FILE "$s\n";
 	close(FILE);
}


# irc stats in RRDtool graphs
#
# count: JOIN: PING: PONGS: PRIVMSGS
#
sub irc_update
{
my $data;

unless (open(HANDLE, $irccount)) {
	irccount_create();
};	
close(HANDLE);

$data = "$t:" . $_[1] . ":" . $_[2] . ":" . $_[3] . ":" .$_[4];

RRDs::update $irccount, $data;
 if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$irccount': $ERROR\n";
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error with rrd/update on $irccount: $ERROR\n";
  };

graph_irccount("irccount.daily-png", "daily: irc stats "." : ".$start_time, "-2000m");
graph_irccount("irccount.weekly-png", "weekly: irc stats "." : ".$start_time, "-12000m");
graph_irccount("irccount.monthly-png", "monthly: irc stats "." : ".$start_time, "-800h");
graph_irccount("irccount.yearly-png", "yearly: irc stats "." : ".$start_time, "-400d");

}

# do not divide by 30 seconds, we want this per period
#
# there are 4 lines: JOINS, PINGS, PONGS, PRIVMSGS
sub graph_irccount
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "count/period",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 

"DEF:joins=$irccount:joins:AVERAGE",
"CDEF:pjoins=joins",

"DEF:pings=$irccount:pings:AVERAGE",
"CDEF:ppings=pings",

"DEF:pongs=$irccount:pongs:AVERAGE",
"CDEF:ppongs=pongs",

"DEF:pmsgs=$irccount:pmsgs:AVERAGE",
"CDEF:ppmsgs=pmsgs",


"LINE1:pjoins#0000ff:joins",
"LINE2:ppings#00ff00:pings",
"LINE3:ppongs#ff0000:pongs",
"LINE1:ppmsgs#0f0f0f:pmsgs",

#"COMMENT:               \\r",
"COMMENT:\\s",
"COMMENT:\\s",
"GPRINT:pjoins:MAX:Max joins %9.0lf %s",
"GPRINT:pjoins:AVERAGE:Average joins %9.0lf %s", 
"GPRINT:pjoins:LAST:Current joins %9.0lf %s\\r", 

"GPRINT:ppings:MAX:Max pings %9.0lf %s",
"GPRINT:ppings:AVERAGE:Average pings %9.0lf %s", 
"GPRINT:ppings:LAST:Current pings %9.0lf %s\\r", 

"GPRINT:ppongs:MAX:Max pongs %9.0lf %s",
"GPRINT:ppongs:AVERAGE:Average pongs %9.0lf %s", 
"GPRINT:ppongs:LAST:Current pongs %9.0lf %s\\r", 

"GPRINT:ppmsgs:MAX:Max pmsgs %9.0lf %s",
"GPRINT:ppmsgs:AVERAGE:Average pmsgs %9.0lf %s", 
"GPRINT:ppmsgs:LAST:Current pmsgs %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
#    die "$0: unable to update `$irccount': $ERROR\n";
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $irccount: $ERROR\n";
#	print "$time, error with rrd/update on $irccount: $ERROR\n";
};

}

sub irccount_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                 zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#               
########################################################################
 
RRDs::create ($irccount, "--start",$start-1, "--step", 30,
        "DS:joins:GAUGE:600:U:U",
        "DS:pings:GAUGE:600:U:U",
        "DS:pongs:GAUGE:600:U:U",
        "DS:pmsgs:GAUGE:600:U:U",

        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",

        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);

my $eirccount = RRDs::error;
	print ERR "rrd creation ERROR : $eirccount\n" if $eirccount;
}

###############################################################################

sub
udpReportOpen
{
	my $pathname = $_[0];
	my $sbuf;
 
	open(udpfd, ">$pathname") || die "can't open $pathname: $!\n";
	print udpfd "topn udp error list: port signatures\n\n";
	$sbuf = sprintf("%-16.16s %-12.12s %-9.9s %8s: %8s: %8s: %9s: %12s: %-5.5s:           %-5.5s:   %9s:  %10s: %s\n\n", 
		"ip src:", 
		"weight:", 
		"guess:",
		"udp_sent", 
		"udp_recv", 
		"unreachs", 
		"ping", 
		"L3D/L4D/L4S", 
		"sizes", 
		"sa/ra", 
		"appflags", 
		"port_count", 
		"port_sig[port, count]");
	print udpfd "$sbuf";
}

sub
udpReport
{
	my $ip = $_[0];
	my $weight = $_[1];
	my $udpsent = $_[2];
	my $udprecv = $_[3];
	my $unreach = $_[4];
	my $icmpcount = $_[5];		
	my $l3count = $_[6];		
	my $l4count = $_[7];		
	my $udpflags = $_[8];		
	my $ports = $_[9];
	my $size0 = $_[10];
	my $size1 = $_[11];
	my $size2 = $_[12];
	my $size3 = $_[13];
	my $size4 = $_[14];
	my $size5 = $_[15];
	my $scount = $_[16];
	my $portstring = $_[17];
	my $savg = $_[18];
	my $ravg = $_[19];
	my @pset;
	my $pindex;
	my $pcount = 0;
	my @sp;
	my $nps="";
	my $sawpossible_dns = 0;

	my $sizeattr = classifySize($size0, $size1, $size2, $size3, $size4, $size5, $udpsent);	
	# format is port,count etc.
     	my @ps = split(/,/,$portstring);

	# first compute the total port count
 	# meaning: the total number of dst sampled packets.	
	# which is NOT necessarily the total number of packets sent.
	for (my $i = 0; $i < $ports; $i++) {
		$pindex = $i * 2;
		$pcount += $ps[$pindex+1];
	}
	# now go thru and compute a percent for the 
	# destination port count (hits/total) and store it in an hash
	# array
	for (my $i = 0; $i < $ports; $i++) {
		$pindex = $i * 2;
		if ($pcount) {
			$pset{$ps[$pindex]} = int(($ps[$pindex+1] * 100.0)/ $pcount);
#print "$pset{$ps[$pindex]}, $ps[$pindex], $ps[$pindex+1]\n";
		}
	}

	# now sort the array based on the port number itself. 
	# sortedPorts is a list of the ports themselves
	@sp  = sort p_by_key keys(%pset);

	# now loop thru the ports
	# putting the ports into one string that is a signature tuple
	for ($i = 0; $i < $ports; $i++) {
		$nps = $nps . "[$sp[$i],$pset{$sp[$i]}]";
		#$ups = $ups . "[$sp[$i]]";			# ports only
	}

	foreach $key (keys(%pset)){
		delete $pset{$key};
	}
	#$portSignature{$ipsrc} = $nps;		# key is ip, value is ports

	# udp guesstimator here
#############################################################
	my $guess = "NA";

# end udp guesstimator
#############################################################

	# print it out
	my $dstring = "$l3count" . "\/" . "$l4count" . "\/" . "$scount";
	my $sizestring2 = "$savg" . "\/" . "$ravg"; 
#print "$dstring\n";
	my $sbuf = sprintf(
		"%-16.16s %-12.12s %-9.9s %9d %9d %9d %9d %14s %-16.16s %-7.7s   %-9.9s      %3d: %s", 
		$ip,      $weight, $guess, $udpsent, $udprecv, $unreach, $icmpcount, 
			$dstring, $sizeattr, $sizestring2, $udpflags, $ports, $nps); 
	print udpfd "$sbuf\n";
	return($sbuf);
}

sub
udpReportClose
{
	close(udpfd);
} 

# sort by the key for the ports array 
# above
sub p_by_key 
{
        if ($a < $b) {
                -1;
        }
        elsif ($a == $b) {
                0;
        }
        elsif ($a > $b) {
                1;
        }
}

# graph the udp IP in terms of its integer weight
#
sub
udpRRD
{
	my $weight = $_[0];

	unless (open(HANDLE, $udpweight)) {
		udpweight_create();
	};	
	close(HANDLE);

	RRDs::update $udpweight, "$t:"."$weight";
 	if ($ERROR = RRDs::error) {
     		$time=`date`;
     		$time=~ s/\n//g;
     		print ERR "$time    error with rrd/update on $udpweight: $ERROR\n";
  	};

	graph_udpweight("udpweight.daily-png", "daily: udp weight"." : ".$start_time, "-2000m");
	graph_udpweight("udpweight.weekly-png", "weekly: udp weight "." : ".$start_time, "-12000m");
	graph_udpweight("udpweight.monthly-png", "monthly: udp weight"." : ".$start_time, "-800h");
	graph_udpweight("udpweight.yearly-png", "yearly: udp weight"." : ".$start_time, "-400d");
}

# don't mod the insert number therefore per period
#
sub graph_udpweight
{
	my $fname = $_[0];
	my $title = $_[1];
	my $stime = $_[2];
 
	RRDs::graph( $fname,
	"--title", $title,
	"-s", $stime,
	"-e",  "now", 
	"-a",  "PNG", 
	"-v" , "weight/period",
	"-w", "500",
	"-h",  "100", 
	"-b", "1000", 
	"--alt-y-grid", 
	"--lazy", 
	"-c", "MGRID#ee0000", 
	"-c", "GRID#000000", 

	"DEF:weight=$udpweight:weight:AVERAGE",
	"CDEF:pweight=weight",
	"LINE1:pweight#0000ff:weight",
	#"COMMENT:               \\r",
	"COMMENT:\\s",
	"COMMENT:\\s",
	"GPRINT:pweight:MAX:Max weight %9.0lf %s",
	"GPRINT:pweight:AVERAGE:Avg weight %9.0lf %s", 
	"GPRINT:pweight:LAST:Last weight %9.0lf %s\\r", 

	);

	if ($ERROR = RRDs::error) {
		$time=`date`;
		$time=~ s/\n//g;
		print ERR "$time, error with rrd/graph on $udpweight: $ERROR\n";
	};

}

sub udpweight_create
{
#######################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a
#                1 data sources, GAUGE type, since they are already
#                 zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                 unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
#               
########################################################################
 
	RRDs::create ($udpweight, "--start",$start-1, "--step", 30,
        	"DS:weight:GAUGE:600:U:U",

		"RRA:AVERAGE:0.5:1:6000",
		"RRA:AVERAGE:0.5:60:700",
		"RRA:AVERAGE:0.5:240:775",
		"RRA:AVERAGE:0.5:2880:797",

		"RRA:MAX:0.5:1:6000",
		"RRA:MAX:0.5:60:700",
		"RRA:MAX:0.5:240:775",
		"RRA:MAX:0.5:2880:797" );

	my $eudpweight = RRDs::error;
	print ERR "rrd creation ERROR : $eudpweight\n" if $eudpweight;
}

sub tcpp2p_create
{
########################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
# 
########################################################################

	RRDs::create ($tcpp2p_proto, "--start",$start-1, "--step",30,
		"DS:totl:GAUGE:600:U:U",
		"DS:tbig:GAUGE:600:U:U",
		"DS:p2ptotl:GAUGE:600:U:U",
		"DS:p2pbig:GAUGE:600:U:U",
		"RRA:AVERAGE:0.5:1:6000",
		"RRA:AVERAGE:0.5:60:700",
		"RRA:AVERAGE:0.5:240:775",
		"RRA:AVERAGE:0.5:2880:797",
		"RRA:MAX:0.5:1:6000",
		"RRA:MAX:0.5:60:700",
		"RRA:MAX:0.5:240:775",
		"RRA:MAX:0.5:2880:797"
	);
	my $etcpp2p_proto = RRDs::error;
	print ERR "rrd creation ERROR : $etcpp2p_proto\n" if $etcpp2p_proto;
}


sub tcpp2p_update
{
	my $total;
	my $totalbig;
	my $p2ptotal;
	my $p2pbig;

	$total = $_[1] + $_[2] + $_[3] + $_[4];
	$totalbig = $_[4];
	$p2ptotal = $_[5] + $_[6] + $_[7] + $_[8];
	$p2pbig = $_[8];

#print "BOO: $_[1], $_[2] total=$total totalbig= $totalbig p2ptotal=$p2ptotal p2pbig=$p2pbig\n";

	RRDs::update $tcpp2p_proto, "$t:"."$total".":"."$totalbig".":"."$p2ptotal".":"."$p2pbig";
	if ($ERROR = RRDs::error) {
	     $time=`date`;
	     $time=~ s/\n//g;
	     print ERR "$time    error on rrd/update for $l2proto: $ERROR\n";
	};

	graph_tcpp2p("tcpp2p.daily-png", "daily: p2p pkt estimate"." : ".$start_time, "-2000m",
		"totl", "tbig", "p2ptotl", "p2pbig");
	graph_tcpp2p("tcpp2p.weekly-png", "weekly: p2p pkt estimate"." : ".$start_time, "-12000m",
		"totl", "tbig", "p2ptotl", "p2pbig");
	graph_tcpp2p("tcpp2p.monthly-png", "monthly: p2p pkt estimate"." : ".$start_time, "-800h",
		"totl", "tbig", "p2ptotl", "p2pbig");
	graph_tcpp2p("tcpp2p.yearly-png", "yearly: p2p pkt estimate"." : ".$start_time, "-400d",
		"totl", "tbig", "p2ptotl", "p2pbig");
}

sub graph_tcpp2p
{

	my $fname = $_[0];
	my $title = $_[1];
	my $stime = $_[2];
	my $label1 = $_[3];
	my $label2 = $_[4];
	my $label3 = $_[5];
	my $label4 = $_[6];

	#print $label1, $label2, $label3, $label4, "\n";
	 
	RRDs::graph( $fname,
	"--title", $title,
	"-s", $stime,
	"-e",  "now", 
	"-a",  "PNG", 
	"-v" , "pkts/sec",
	"-w", "500",
	"-h",  "100", 
	"-b", "1000", 
	"--alt-y-grid", 
	"--lazy", 
	"-c", "MGRID#ee0000", 
	"-c", "GRID#000000", 
	#  "DEF:ucast=$cast:ucast:AVERAGE",
	#  "DEF:bcast=$cast:bcast:AVERAGE",
	#  "DEF:xtra=$cast:xtra:AVERAGE",

	"DEF:totl=$tcpp2p_proto:totl:AVERAGE",
	"CDEF:ptotl=totl,30,/",

	"DEF:tbig=$tcpp2p_proto:tbig:AVERAGE",
	"CDEF:ptbig=tbig,30,/",

	"DEF:p2ptotl=$tcpp2p_proto:p2ptotl:AVERAGE",
	"CDEF:pp2ptotl=p2ptotl,30,/",

	"DEF:p2pbig=$tcpp2p_proto:p2pbig:AVERAGE",
	"CDEF:pp2pbig=p2pbig,30,/",

	"LINE1:ptotl#0000ff:$label1",
	"LINE1:ptbig#00ff00:$label2",
	"LINE1:pp2ptotl#ff0000:$label3",
	"LINE1:pp2pbig#ff00ff:$label4",

	"COMMENT:\\s",
	"COMMENT:\\s",
#totl
#tbig
#p2ptotl
#p2pbig

	"GPRINT:ptotl:MAX:Max totl %9.0lf %s",
	"GPRINT:ptotl:AVERAGE:Average totl %9.0lf %s", 
	"GPRINT:ptotl:LAST:Current totl %9.0lf %s\\r", 

	"GPRINT:ptbig:MAX:Max tbig %9.0lf %s",
	"GPRINT:ptbig:AVERAGE:Average tbig %9.0lf %s", 
	"GPRINT:ptbig:LAST:Current tbig %9.0lf %s\\r", 

	"GPRINT:pp2ptotl:MAX:Max p2ptotl %9.0lf %s",
	"GPRINT:pp2ptotl:AVERAGE:Average p2ptotl %9.0lf %s", 
	"GPRINT:pp2ptotl:LAST:Current p2ptotl %9.0lf %s\\r", 

	"GPRINT:pp2pbig:MAX:Max p2pbig %9.0lf %s",
	"GPRINT:pp2pbig:AVERAGE:Average p2pbig %9.0lf %s", 
	"GPRINT:pp2pbig:LAST:Current p2pbig %9.0lf %s\\r", 
	);

	if ($ERROR = RRDs::error) {
		$time=`date`;
		$time=~ s/\n//g;
		print ERR "$time, error with rrd/graph on $tcpp2p_proto: $ERROR\n";
	} 
}


sub l2proto_create
{
########################################################################
#
#  $start-1
#                the time slot for starting
#  "--step", 30
#                data updated for every 30 seconds
#  DS:a, DS:b, DS:c, DS:d
#                4 data sources, all is GAUGE type, since they are already
#                zeroed for every 30 seconds
#  600:U:U
#                if no data update for 600 seconds, data will be shown as
#                unknown. U:U means no max:min limit
#  0.1:1:2880
#                sample data for every 30 seconds, 30 * 2880 = 24 hours
# 
########################################################################

RRDs::create ($l2proto, "--start",$start-1, "--step",30,
        "DS:ip:GAUGE:600:U:U",
        "DS:ipv6:GAUGE:600:U:U",
        "DS:arp:GAUGE:600:U:U",
        "DS:xtra:GAUGE:600:U:U",
        "RRA:AVERAGE:0.5:1:6000",
        "RRA:AVERAGE:0.5:60:700",
        "RRA:AVERAGE:0.5:240:775",
        "RRA:AVERAGE:0.5:2880:797",
        "RRA:MAX:0.5:1:6000",
        "RRA:MAX:0.5:60:700",
        "RRA:MAX:0.5:240:775",
        "RRA:MAX:0.5:2880:797"
);
my $el2proto = RRDs::error;
print ERR "rrd creation ERROR : $el2proto\n" if $el2proto;
}

sub l2proto_update
{
RRDs::update $l2proto, "$t:"."$_[2]".":"."$_[4]".":"."$_[6]".":"."$_[8]";
if ($ERROR = RRDs::error) {
     $time=`date`;
     $time=~ s/\n//g;
     print ERR "$time    error on rrd/update for $l2proto: $ERROR\n";
};

graph_l2proto("l2proto.daily-png", "daily: Fixed L2 proto pkts"." : ".$start_time, "-2000m",
	$_[1], $_[3], $_[5], $_[7]);
graph_l2proto("l2proto.weekly-png", "weekly: Fixed L2 proto pkts"." : ".$start_time, "-12000m",
	$_[1], $_[3], $_[5], $_[7]);
graph_l2proto("l2proto.monthly-png", "monthly: Fixed L2 proto pkts"." : ".$start_time, "-800h",
	$_[1], $_[3], $_[5], $_[7]);
graph_l2proto("l2proto.yearly-png", "yearly: Fixed L2 proto pkts"." : ".$start_time, "-400d",
	$_[1], $_[3], $_[5], $_[7]);
}

sub graph_l2proto
{

my $fname = $_[0];
my $title = $_[1];
my $stime = $_[2];
my $label1 = $_[3];
my $label2 = $_[4];
my $label3 = $_[5];
my $label4 = $_[6];

#print $label1, $label2, $label3, $label4, "\n";
 
RRDs::graph( $fname,
"--title", $title,
"-s", $stime,
"-e",  "now", 
"-a",  "PNG", 
"-v" , "pkts/sec",
"-w", "500",
"-h",  "100", 
"-b", "1000", 
"--alt-y-grid", 
"--lazy", 
"-c", "MGRID#ee0000", 
"-c", "GRID#000000", 
#  "DEF:ucast=$cast:ucast:AVERAGE",
#  "DEF:bcast=$cast:bcast:AVERAGE",
#  "DEF:xtra=$cast:xtra:AVERAGE",

"DEF:ip=$l2proto:ip:AVERAGE",
"CDEF:pip=ip,30,/",

"DEF:ipv6=$l2proto:ipv6:AVERAGE",
"CDEF:pipv6=ipv6,30,/",

"DEF:arp=$l2proto:arp:AVERAGE",
"CDEF:parp=arp,30,/",

"DEF:xtra=$l2proto:xtra:AVERAGE",
"CDEF:pxtra=xtra,30,/",

"LINE1:pip#0000ff:$label1",
"LINE1:pipv6#00ff00:$label2",
"LINE1:parp#ff0000:$label3",
"LINE1:pxtra#ff00ff:$label4",

"COMMENT:\\s",
"COMMENT:\\s",

"GPRINT:pip:MAX:Max ip %9.0lf %s",
"GPRINT:pip:AVERAGE:Average ip %9.0lf %s", 
"GPRINT:pip:LAST:Current ip %9.0lf %s\\r", 

"GPRINT:pipv6:MAX:Max ipv6 %9.0lf %s",
"GPRINT:pipv6:AVERAGE:Average ipv6 %9.0lf %s", 
"GPRINT:pipv6:LAST:Current ipv6 %9.0lf %s\\r", 

"GPRINT:parp:MAX:Max arp %9.0lf %s",
"GPRINT:parp:AVERAGE:Average arp %9.0lf %s", 
"GPRINT:parp:LAST:Current arp %9.0lf %s\\r", 

"GPRINT:pxtra:MAX:Max xtra %9.0lf %s",
"GPRINT:pxtra:AVERAGE:Average xtra %9.0lf %s", 
"GPRINT:pxtra:LAST:Current xtra %9.0lf %s\\r", 
);

if ($ERROR = RRDs::error) {
      	$time=`date`;
      	$time=~ s/\n//g;
 	print ERR "$time, error with rrd/graph on $l2proto: $ERROR\n";
} 

}

########################################################################3
# command-line functions

sub printUsageAndExit()
{
	print "-m mon.lite_file (default: /usr/home/mrourmon/monfiles/mon.lite)\n";
	print "-l logging_dir (default: /usr/home/mrourmon/logs)\n";
	print "-r rrdtool_database_dir (default: /usr/home/mrourmon/rrdbase)\n";
	print "-w web_dir_path  (default: /usr/local/apache/htdocs/ourmon)\n";
	print "-L [on | off]    (default: on)\n";
	print "-a megabits      (default: 10000000)\n"; 
	print "-u udpweight     (default: 10000000)\n"; 
	print "-U dir path to udpreport.txt (default: web_dir_path)\n"; 
	print "-e eventfile\n";
	print "-h :help message\n";
	exit 1;
}

# debug function - dump various command-line args
# 	and any other important variables
sub dumpArgs()
{
	print "(FILE)datafile - $datafile\n";
	print "(DIR)rrdbase - $rrdbase\n";
	print "(VAR)logbase - $logbase\n";
	print "(DIR)webpath - $webpath\n";
	print "(DIR)eventfile - $eventfile\n";
	print "(VAR)logging - $logging\n";
	print "(VAR)maxalertsize - $maxalertsize\n";
	print "(VAR)maxudpweight - $maxudpweight\n";
}

# -m mon.lite file (absolute dir/filename)
# -w web_dir path (absolute dir)
# -r rrdtool database path (absolute)
# -t absolute path to drawtopn program
# -l logging base directory path (absolute)
# -L logging on, else off.  args are {on, off}
# -a N  alert size in bits
# -z print defaults and exit
# -u udpweight
sub processCmdArgs
{
  	my $ret = 0;
  	my $arg;
  	while($arg = shift(@ARGV)){
    		if($arg eq '-z'){
			dumpArgs();
			exit(1);
    		}
		# mon.lite file
    		if($arg eq '-m'){
      			$datafile = '';   # clear the default config file
      			$datafile =  shift(@ARGV);
      			if($datafile eq '' || !defined($datafile)){
         			printUsageAndExit();
      			}
    		}
		# web dir path
		elsif($arg eq '-w'){
			$webpath = '';
			$webpath = shift(@ARGV);
			if($webpath eq '' || !defined($webpath)){
				printUsageAndExit();
		      	}
		}
		# rrdbase path
		elsif($arg eq '-r'){
			$rrdbase = '';
			$rrdbase = shift(@ARGV);
			if($rrdbase eq '' || !defined($rrdbase)){
				printUsageAndExit();
		      	}
		}
		# logbase
		elsif($arg eq '-l'){
		      $logbase = '';
		      $logbase = shift(@ARGV);
		      if($logbase eq '' || !defined($logbase)){
				printUsageAndExit();
		      }
		}
		# logging on/off
		elsif($arg eq '-L'){
		      $logging = '';
		      $logging = shift(@ARGV);
		      if($logging eq '' || !defined($logging)){
				printUsageAndExit();
		      }
		}
		# udpweight
		elsif($arg eq '-u'){
		      $maxudpweight = '';
		      $maxudpweight = shift(@ARGV);
		      if($maxudpweight eq '' || !defined($maxudpweight)){
				printUsageAndExit();
		      }
		}
		# path to udpreport.txt
		elsif($arg eq '-U'){
		      $udpreport_base = shift(@ARGV);
		}
		# alertsize
		elsif($arg eq '-a'){
		      $maxalertsize = '';
		      $maxalertsize = shift(@ARGV);
		      if($maxalertsize eq '' || !defined($maxalertsize)){
				printUsageAndExit();
		      }
		}
		elsif($arg eq '-e'){
		      $eventfile = '';
		      $eventfile = shift(@ARGV);
		      if($eventfile eq '' || !defined($eventfile)){
				printUsageAndExit();
		      }
		      $eventflag = 1;
		}
		# help or anything else is help
    		elsif($arg eq '-h'){
      			printUsageAndExit();
    		}
    		else{
      			printUsageAndExit();
    		}
	}
}

#####################
# used for all topn type tuples
# TBD 1: check what happens when there are 0 tuples
# TBD 2: enhance topn_icmp flows to translate major/minor
#	code from integer to ASCII english tag (ttl exceeded, etc.)
# tuple used:
#	FLOWTUPLE
#	PORTTUPLE
#	SYNTUPLE
#	UDPTUPLE 
#
# topn_bar
#	handles all topn lists 
#
sub topn_bar
{
	# reference for all the function args into items
	my(@items) = @{(shift)};
	my $div = 1; 
	my $total_flows; 
	my $most_wide;
	my %flows;
	my $width; 
	my $tall = 25;
	# used to normalize the width so that the largest count
	# is N pixels wide:
	# don't change this unless you change the bar graph file size
	my $max_pixels_wide = 950;
	
	my $output ="<html>\n<head>\n<title>$title</title>\n" . 
				"</head>\n<body bgcolor=\"#FFFFFF\">\n"; 
				#. "<b>$title report for <a name=data></a>$start_time\n</b>\n<p>";
	$title= $items[0];
	$total_flows = $items[1];


	$count = $items[3];	
	# get largest item in list, whatever that might be
	# this is true for syn_list
	# 3rd item should be sorted by value

	# return number of items per tuple
	my $inc = get_index_inc($items[0]);

	# special case udperror_list for opening and closing
	# as we need to generate the udp portreport and RRD top weight
	if ($items[0] eq "udperror_list") {
		my $udpfile=$udpreport_base . "\/" . "udpreport.txt";
		udpReportOpen($udpfile);
	}

	# check for no data items first
	if ( $items[1] eq 0) {
		$output = "$output"  . "no data items this sample";
	}
	else {
		# traverse the mon.lite tuples 
		# i is for list traversal
		# k makes the bar graph file unique
		# 
		for (my $i = 2, my $k = 0; $i < $#items; $i+=$inc, $k++) {
			$output = "$output" . topn_html(\@items,$i,$k,$count);
		}
	}

	# now close udp port report if necessary
	if ($items[0] eq "udperror_list") {
		udpReportClose();
	}
	# put on sacred end html file garp
	$output = "$output\n</body>\n</html>";
		
	# name of 2ndary html file
	my $outfile = "$title" . ".html";
	
	# print the bleeding file
	open(HTML,">$outfile") or die "Cannot open $outfile";
	print HTML $output;
	close(HTML);
}


#####################################
# depending on type of tuple, output corresponding html
#
# per tuple brains live here!
#
# input is entire array:
#	tuple is indexed by i
#	k is file extension number
sub topn_html 
{
	my(@items) = @{(shift)};
	my $i = shift;
	my $k = shift;
	my $maxcount = shift;
	my $index;
	
	# scale the current sorted by value by the max value (top N)
	# must range 0..100
	if ($maxcount < 1) {
		$maxcount = 1;
	}
        $index = int(($items[$i+1] * 100.0)/$maxcount);  
	if ($index < 1) {
		$index = 1;
	}
	$fname =  "$barname/bar$index.png";

	if ( $items[0] eq "topn_ip_pkts") {
		my $uappf = $items[$i+2];	# old hw app flags
		my $uappl = $items[$i+3];	# app lower case
		my $uapph = $items[$i+4];	# app upper case
		my $pps = int($items[$i+1]/30);
		my $uaf = get_appflag($uappf, $uappl, $uapph);

		return 	"<b>flow: </b>$items[$i],  " .
				"<b>pkts/sec:</b> $pps, flags: $uaf<br>" .
				"<img src=\"$fname\"><p>\n";
	}
	if ( $items[0] =~ /topn/ ) {
		my $uappf = $items[$i+2];	# old hw app flags
		my $uappl = $items[$i+3];	# app lower case
		my $uapph = $items[$i+4];	# app upper case
		my $bps = int(($items[$i+1] * 8)/30);
		my $uaf = get_appflag($uappf, $uappl, $uapph);


		#return 	"<b>flow: </b>$items[$i],  " .
		#		"<b>bits/sec:</b> $items[$i+1]<br>" .
		#		"<img src=\"$barname/$items[0]$k.png\"><p>\n";
		return 	"<b>flow: </b>$items[$i],  " .
				"<b>bits/sec:</b> $bps, flags: $uaf<br>" .
				"<img src=\"$fname\"><p>\n";
	}
	elsif ($items[0] eq "tcp_ports"
		|| $items[0] eq "udp_ports") {
		my $bps = int(($items[$i+1] * 8)/30);

		my $uappf = $items[$i+4];	# old hw app flags
		my $uappl = $items[$i+5];	# app lower case
		my $uapph = $items[$i+6];	# app upper case
		my $uaf = get_appflag($uappf, $uappl, $uapph);

		return 	"<b>port: </b>$items[$i], " .
				"<b>bits/sec: </b> $bps, " .
				"<b>src: </b> $items[$i+2], " .
				"<b>dst: </b> $items[$i+3]," .
				"<b>flags: </b> $uaf<br>" .
				"<img src=\"$fname\"><p>\n";
	}
	elsif ($items[0] eq "tcp_portscan"
		|| $items[0] eq "udp_portscan"
		|| $items[0] eq "ip_portscan" 
		|| $items[0] eq "ip_scan" ) {
		my $ip = $items[$i];
		my $count = $items[$i+1];
		my $uappf = $items[$i+2];	# old hw app flags
		my $uappl = $items[$i+3];	# app lower case
		my $uapph = $items[$i+4];	# app upper case
		my $uaf = get_appflag($uappf, $uappl, $uapph);
		my $label = "L4 dsts";
		if ($items[0] eq "ip_scan") {
			$label = "ip dsts";
		}
		return 		"<b>scanner: </b>$ip,  " .
				"<b>$label: </b> $count," .
				"<b>flags: </b> $uaf<br>" .
				"<img src=\"$fname\"><p>\n";
	}
	# udperror_list tuple syntax:
	# update and make png file for topn_udperror
	# format: ip address: weight: udp sends: udp recvs: unreach: pings: l3:       l4: udpflags: noports: port tuple (dst_port, count)
	#	  0	      1       2          3          4		5     6        7        8   9	     10
	elsif ($items[0] eq "udperror_list") {
		my $udps;
		my $udpip  = $items[$i];
		my $udpww  = $items[$i+1];
		my $udpsent = $items[$i+2];
		my $udprecv = $items[$i+3];
		my $unreach = $items[$i+4];	# icmp unreach sent back
		my $icmpcount = $items[$i+5];	# pings sent by ip
		my $l3count   = $items[$i+6];	# count of IP dsts
		my $l4count   = $items[$i+7];	# count of UDP dsts
		my $uappf = $items[$i+8];	# old hw app flags
		my $uappl = $items[$i+9];	# app lower case
		my $uapph = $items[$i+10];	# app upper case
		my $size0 = $items[$i+11];	
		my $size1 = $items[$i+12];	
		my $size2 = $items[$i+13];	
		my $size3 = $items[$i+14];	
		my $size4 = $items[$i+15];	
		my $size5 = $items[$i+16];	

		my $savg = $items[$i+17];	
		if ($udpsent) {
			$savg = int($savg/$udpsent);
	 	}
		else {
			$savg = 0;
		}
		my $ravg = $items[$i+18];	
		if ($udprecv) {
			$ravg = int($ravg/$udprecv);
	 	}
		else {
			$ravg = 0;
		}

		my $portcount = $items[$i+19];	# number of port tuples
		my $scount = $items[$i+20];	# number of port tuples
		my $portstring = $items[$i+21]; # all port tuples in one string
		my $uaf = get_appflag($uappf, $uappl, $uapph);

		# first take care of udpreport.txt output - one line per tuple
		if ($udpww > 0) {
			$udps = udpReport($udpip, $udpww, $udpsent, 
				$udprecv, $unreach, $icmpcount, 
				$l3count, $l4count, $uaf, $portcount, 
				$size0, $size1, $size2, $size3, $size4, $size5, $scount,
				$portstring, $savg,$ravg);
			if ($evilflag) {
				storeEvent($eventfile, "udp evil flag:", $udps);
			}
			if ($global_evil) {
				storeEvent($eventfile, "udp/stormworm???:", $udps);
			}
		}
		if ($k == 0) {
			udpRRD($udpww);
			if ($udpww > $maxudpweight) {
				storeEvent($eventfile, 
					"udpweight threshold exceeded:", $udps);				
			}
		}
		# now produce an html per tuple output 
		return 	"<b>ip: </b>$udpip,  " .
				"<b>weight/period:</b>  $udpww, " .
"Snd: $udpsent, Rcv: $udprecv, Unr: $unreach, PNG: $icmpcount, L3D/L4D: $l3count/$l4count, " .
				"<b>flags:</b> $uaf <br>" .
				"<img src=\"$fname\"><p>\n";
	}

	# icmperror_list tuple syntax:
	# update and make png file for topn_icmperror
	# format: ip address: icmp error count: tcp pkt count: udp count: icmp count: unreach: redirect: ttl: udpflags
	# note: icmp count here are ping (echo request) only
	# e.g., 131.252.243.64:238:3258:4:238:1:0:237: 0
	#       0              1   2    3 4   5 6 7    8
	# T - tcp count
	# U - udp
	# E - ping total
	# Un - unreachables
	# R - redirects
	# X - time exceeded
	# P - "pot", darknet pkts 
	elsif ($items[0] eq "icmperror_list") {
		# note: $items[$i] is the ip address
		my $icmps = $items[$i+1];	#  sorted by error count
		my $tcps = $items[$i+2];	# 
		my $udps = $items[$i+3];	# 
		my $pings = $items[$i+4];	# 
		my $unreach = $items[$i+5];	# 
		my $redirect = $items[$i+6];	# 
		my $ttl = $items[$i+7];	# 
		my $udpflags = $items[$i+8];	# 
		if ($udpflags & 0x01) {
			my $flags = "P";
		}
		return 	"<b>ip: </b>$items[$i],  " .
				"<b>icmps/period:</b>  $icmps, " .
				"Tcp: $tcps, Udp: $udps, Pings: $pings, Unr: $unreach, Red: $redirect, ttlx: $ttl, " .

				"<b>flags:</b> $flags<br>" .
				"<img src=\"$fname\"><p>\n";
	}
	# syn_list tuple syntax:
	# ip: syn: synack: fins(returned): finssent: rst: tcpfrom: tcpto:
# icmpcount: appflags: apptags.lower : apptags.upper: ehits: ipds: tcpds: tcpsps: one src port:  Nports: port,count,port,count,port,count
	#   0    1    2     3               4         5     6       7       8
#   9  10     11      12   13      14   15 16 17 18
	#   65.45.204.94:9466:0:9121:9136:916:45992:37428:0:0:0:0:0:10:1500:1:1433,45992,:
	elsif ($items[0] eq "syn_list") {
		my $syns = $items[$i+1];	# syns sent by IP
		my $fins = $items[$i+3];	# fins returned to ip
		my $resets = $items[$i+5];	# resets returned to ip
		my $tcpfrom = $items[$i+6];	# tcp pkts sent by ip
		my $tcpto = $items[$i+7];	# tcp pkts returned to ip
		my $appf = $items[$i+9];	# old hw app flags
		my $appl = $items[$i+10];	# app lower case
		my $apph = $items[$i+11];	# app upper case
		my $af = get_appflag($appf, $appl, $apph);
		my $total = $tcpfrom + $tcpto;  #
		my $ww = calc_ww($syns, $fins, $resets, $total);
		return 	"<b>ip: </b>$items[$i],  " .
				"<b>per period:</b> s:$syns, f:$fins, r:$resets, " .
				"<b>total:</b> $total, " .
				"<b>ww:</b> $ww, apps: $af<br>" .
				"<img src=\"$fname\"><p>\n";
	}
	else {
		print "Error in omupdate.pl/topn_html()\n";
		exit(1);
	}
}

##################################
# items in top n list are different depending on tuple count,
# this function determines how we walk the tuple list
# returns: per tuple field count
sub get_index_inc
{
	my $name = shift;

	if ( $name eq "tcp_ports" ) {
		return 7;
	}
	elsif ( $name eq "udp_ports" ) {
		return 7;
	}
	elsif ( $name eq "icmperror_list" ) {
		return 9;
	} 
	elsif ( $name eq "udperror_list" ) {
		return 22;
	} 
	elsif ( $name eq "syn_list" ) {
		return 26;
	}
	elsif ( $name =~ /topn/ ) {
		return 5;
	}
	elsif ($name eq "tcp_portscan") {
		return 5;
	}
	elsif ($name eq "udp_portscan") {
		return 5;
	}
	elsif ($name eq "ip_portscan") {
		return 5;
	}
	elsif ($name eq "ip_scan") {
		return 5;
	}
	else {
		return 2;
	}
}


sub calc_ww
{
	my $syns = $_[0];
	my $fins = $_[1];
	my $rsts = $_[2];
	my $total = $_[3];

	my $percent = 0;

	my $tcpc = $syns + $fins + $rsts;	# total control packets
	if ($total) {
		$percent = int(($tcpc * 100.0)/$total);
	}
	# tack on percent/worm opinion and concat count to get label/count for 
	# drawtopn
	return("$percent%");
}

# parse syn_list app flags
sub get_appflag 
{
	my $appflags = $_[0];
	my $apptagsLower = $_[1];
	my $apptagsUpper = $_[2];
	my $emulehits = 1;

	my $af = parseAppFlags(hex($appflags), $emulehits);
	$af = parseLowerAppTags(hex($apptagsLower),$af);
	$af = parseUpperAppTags(hex($apptagsUpper),$af);
	return($af);
}

# ignoring emulehit count for now
sub parseAppFlags
{
	$appflags = $_[0];
	$emulehits = $_[1];
	#$appflags = hex($_[0]);
	my $string="";

#### hack
	$evilflag = 0;
####

	if ($appflags & 0x0f) {
		$string = "I";		#/* irc of some sort */
	}
	if ($appflags & 0x10) {
		$string .= "B";		#/* bittorrent */
	}
	if ($appflags & 0x20) {
		$string .= "G";		#/* GNUTELLA */
	}
	if ($appflags & 0x40) {
		$string .= "K";		#/* KAZAA */
	}
	if ($appflags & 0x80 && ($emulehits >= $emulecount)) {
		$string .= "e";		#/* edonkey */
	}
 	if ($appflags & 0x100) {
 		$string .= "P";		#/* honeypot dst */
 	}
	if ($appflags & 0x200) {
		$string .= "M";		#/* Morpheus */
	}
	if ($appflags & 0x400) {
		$string .= "H";		#/* web server based on tcp src port */
	}
	if ($appflags & 0x800) {
		$string .= "E";		# email based on tcp dst port 
	}
	if ($appflags & 0x1000) {
		$string .= "D";		# darknet dst
	}
	if ($appflags & 0x2000) {
		$string .= "X";		# evil flag
		$evilflag = 1;
	}
	return($string);
}

sub parseLowerAppTags 
{
	$apptag = $_[0];
	$string = $_[1];

	if ($apptag & 0x1 ) { $string .= "a"; }
	if ($apptag & 0x2 ) { $string .= "b"; }
	if ($apptag & 0x4 ) { $string .= "c"; }
	if ($apptag & 0x8 ) { $string .= "d"; }
	if ($apptag & 0x10 ) { $string .= "e"; }
	if ($apptag & 0x20 ) { $string .= "f"; }
	if ($apptag & 0x40 ) { $string .= "g"; }
	if ($apptag & 0x80 ) { $string .= "h"; }
	if ($apptag & 0x100 ) { $string .= "i"; }
	if ($apptag & 0x200 ) { $string .= "j"; }
	if ($apptag & 0x400 ) { $string .= "k"; }
	if ($apptag & 0x800 ) { $string .= "l"; }
	if ($apptag & 0x1000 ) { $string .= "m"; }
	if ($apptag & 0x2000 ) { $string .= "n"; }
	if ($apptag & 0x4000 ) { $string .= "o"; }
	if ($apptag & 0x8000 ) { $string .= "p"; }
	if ($apptag & 0x10000 ) { $string .= "q"; }
	if ($apptag & 0x20000 ) { $string .= "r"; }
	if ($apptag & 0x40000 ) { $string .= "s"; }
	if ($apptag & 0x80000 ) { $string .= "t"; }
	if ($apptag & 0x100000 ) { $string .= "u"; }
	if ($apptag & 0x200000 ) { $string .= "v"; }
	if ($apptag & 0x400000 ) { $string .= "w"; }
	if ($apptag & 0x800000 ) { $string .= "x"; }
	if ($apptag & 0x1000000 ) { $string .= "y"; }
	if ($apptag & 0x2000000 ) { $string .= "z"; }

	return($string);
}

sub parseUpperAppTags 
{
	$apptag = $_[0];
	$string=  $_[1];

	if ($apptag & 0x1 ) { $string .= "A"; }
	if ($apptag & 0x2 ) { $string .= "B"; }
	if ($apptag & 0x4 ) { $string .= "C"; }
	if ($apptag & 0x8 ) { $string .= "D"; }
	if ($apptag & 0x10 ) { $string .= "E"; }
	if ($apptag & 0x20 ) { $string .= "F"; }
	if ($apptag & 0x40 ) { $string .= "G"; }
	if ($apptag & 0x80 ) { $string .= "H"; }
	if ($apptag & 0x100 ) { $string .= "I"; }
	if ($apptag & 0x200 ) { $string .= "J"; }
	if ($apptag & 0x400 ) { $string .= "K"; }
	if ($apptag & 0x800 ) { $string .= "L"; }
	if ($apptag & 0x1000 ) { $string .= "M"; }
	if ($apptag & 0x2000 ) { $string .= "N"; }
	if ($apptag & 0x4000 ) { $string .= "O"; }
	if ($apptag & 0x8000 ) { $string .= "P"; }
	if ($apptag & 0x10000 ) { $string .= "Q"; }
	if ($apptag & 0x20000 ) { $string .= "R"; }
	if ($apptag & 0x40000 ) { $string .= "S"; }
	if ($apptag & 0x80000 ) { $string .= "T"; }
	if ($apptag & 0x100000 ) { $string .= "U"; }
	if ($apptag & 0x200000 ) { $string .= "V"; }
	if ($apptag & 0x400000 ) { $string .= "W"; }
	if ($apptag & 0x800000 ) { $string .= "X"; }
	if ($apptag & 0x1000000 ) { $string .= "Y"; }
	if ($apptag & 0x2000000 ) { $string .= "Z"; }

	return($string);
}

# returns one string with all sizes 
sub classifySize
{
	$size0=$_[0];
	$size1=$_[1];
	$size2=$_[2];
	$size3=$_[3];
	$size4=$_[4];
	$size5=$_[5];
	$udpsent=$_[6];
	my $string="";
	my $s0, $s1, $s2, $s3, $s4, $s5;

	#return("$size0/$size1/$size2/$size3");
	# convert to percent
	$s0 = $s1 = $s2 = $s3 = $s4 = $s5 = 0;
	if ($udpsent) {
		$s0 = int(($size0*100)/$udpsent);
		$s1 = int(($size1*100)/$udpsent);
		$s2 = int(($size2*100)/$udpsent);
		$s3 = int(($size3*100)/$udpsent);
		$s4 = int(($size4*100)/$udpsent);
		$s5 = int(($size5*100)/$udpsent);
	}
	if ($s0 > 100) { $s0 = 100;}
	if ($s1 > 100) { $s1 = 100;}
	if ($s2 > 100) { $s2 = 100;}
	if ($s3 > 100) { $s3 = 100;}
	if ($s4 > 100) { $s4 = 100;}
	if ($s5 > 100) { $s5 = 100;}
	# returns one string
	return("$s0/$s1/$s2/$s3/$s4/$s5");
}
