#!/bin/sh
# omupdate.sh
#
# ourmon - backend graphics/logging driver.
# called by crontab once per minute

# these may change - logical params
BASEDIR=/home/mrourmon
FLOWSIZE=10000000
UDPWEIGHT=10000000

##########mods for remote file fetch #########
# if you want to use wget to fetch files remotely
# comment these two variables in and set the 
# PROBE IP to something useful
# you should also check out the location of wget
# below and modify if necessary

#USEWGET="yes"
#PROBE=127.0.0.1

#turn on if wget used with probe to fetch probe output files
#note timeout so wget will give up and not pileup due to cron
#thus one can avoid having 1000 wgets trying to run to a probe
#that is down for some reason
# tries/connect-timeout may need adjusting
# make sure the path for wget is correct
if [ $USEWGET ]; then
WGET="/usr/local/bin/wget --tries=4 --connect-timeout=4 "
fi
##########mods for remote file fetch #########

#################################################### 
# everthing else is derived from BASEDIR
BINDIR=$BASEDIR/bin
RRDDATA=$BASEDIR/rrddata
# we assume this is a symlink to the real web directory
WEBDIR=$BASEDIR/web.pages
LOGDIR=$BASEDIR/logs
TMPDIR=$BASEDIR/tmp

# input files
# position of probe outputs, inputs for graphics-engine
MONLITE=$TMPDIR/mon.lite
TCPWORM=$TMPDIR/tcpworm.txt
P2PFILE=$TMPDIR/p2pfile.txt
IRCFILE=$TMPDIR/irc.txt
SYNDUMP=$TMPDIR/syndump.txt
EMAILDUMP=$TMPDIR/emaildump.txt
# on although possibly not useful
POTDUMP=$TMPDIR/potdump.txt

# do twice, once per 30 seconds
for i in 1 2
do

# first get all the files from the probe

if [ $USEWGET ]; then
$WGET -q --output-document=$MONLITE http://$PROBE/mon.lite
SUCCESS=$?
# bail on this run if we can't get probe files for some reason
if [ $SUCCESS -ne 0 ]; then
	echo "`date`: failure to fetch probe-files" >> $WEBDIR/event_today.txt
	exit 1
fi
$WGET -q --output-document=$TCPWORM http://$PROBE/tcpworm.txt
$WGET -q --output-document=$P2PFILE http://$PROBE/p2pfile.txt
$WGET -q --output-document=$IRCFILE http://$PROBE/irc.txt
$WGET -q --output-document=$SYNDUMP http://$PROBE/syndump.txt
$WGET -q --output-document=$EMAILDUMP http://$PROBE/emaildump.txt
$WGET -q --output-document=$POTDUMP http://$PROBE/potdump.txt
fi

# LOG backup of mon.lite to logs/mon.lite directory
$BINDIR/monbackup.pl $MONLITE $LOGDIR/mon.lite

# LOG daily/topn_log dirs
# run omupdate.pl graph/report maker with appropriate parameters
$BINDIR/omupdate.pl -m $MONLITE -w $WEBDIR -r $RRDDATA -l $LOGDIR -a $FLOWSIZE -u $UDPWEIGHT -e $WEBDIR/event_today.txt
ETIME=$?

# extract any config info from mon.lite which is used elsewhere
# in the back-end.  this allows us to avoid having to configure
# the back-end for knowledge of home networks

HOMECONFIG=`grep "^#HOMENETWORK" $MONLITE | cut -d : -f 2` 
echo $HOMECONFIG > $LOGDIR/homeconfig.txt
OURNET=$HOMECONFIG

# LOG udpreport
# udpreport.txt is made as a side effect of the omupdate.pl run.
# if udpreport.txt exists, copy it to a log directory
if [ -f $WEBDIR/udpreport.txt ]; then
	$BINDIR/monbackup.pl $WEBDIR/udpreport.txt $LOGDIR/udpreport
fi

# LOG tworm 
# backup of tcpworm.txt to logs/tworm directory
$BINDIR/monbackup.pl $TCPWORM $LOGDIR/tworm
# run tcpworm.pl with the appropriate parameters
$BINDIR/tcpworm.pl  -E $WEBDIR/event_today.txt -S $WEBDIR/newportsig_today.txt -p $WEBDIR/portreport.txt  -D $LOGDIR/topn_today/scanner $TCPWORM > $WEBDIR/tcpworm.txt

# LOG portreport
# use monbackup.pl to backup the portreport.txt output file
$BINDIR/monbackup.pl $WEBDIR/portreport.txt $LOGDIR/portreport

# run tcpworm.pl again to produce p2p-based portreport with app flags in it
$BINDIR/tcpworm.pl -t -p $WEBDIR/p2pportreport.txt  $P2PFILE > $WEBDIR/p2pfile.txt
TTIME=$?

# run tcpworm.pl again to produce syndump portreport with app flags in it
$BINDIR/tcpworm.pl -t -p $WEBDIR/synreport.txt  $SYNDUMP > $WEBDIR/synreport_long.txt
SYNTIME=$?
$BINDIR/monbackup.pl $SYNDUMP $LOGDIR/rawsyndump

# emaildump
$BINDIR/tcpworm.pl -e 1000 -p $WEBDIR/emailreport.txt $EMAILDUMP > $WEBDIR/emailreport_long.txt
$BINDIR/monbackup.pl $EMAILDUMP $LOGDIR/rawemail

# potdump
# configured to write to event log only if home address
# is found in potdump file
if [ "$OURNET" ]; then
$BINDIR/tcpworm.pl -R $OURNET -E $WEBDIR/event_today.txt -p $WEBDIR/potdump.txt $POTDUMP > $WEBDIR/potdump_long.txt
else
$BINDIR/tcpworm.pl -p $WEBDIR/potdump.txt $POTDUMP > $WEBDIR/potdump_long.txt
fi

$BINDIR/monbackup.pl $POTDUMP $LOGDIR/rawpotdump

# LOG p2p files raw and processed
$BINDIR/monbackup.pl $P2PFILE $LOGDIR/rawp2p
$BINDIR/monbackup.pl $WEBDIR/p2pportreport.txt $LOGDIR/p2preport

# LOG daily/syn_worm.log
# convert the tcpworm.txt file into a log entry for batch processing later on
# this is gross thing and should go away eventually.
$BINDIR/wormtolog.pl $TCPWORM  $LOGDIR  syn_worm.log

# LOG irc
# now do irc report and backup of output
$BINDIR/irc.pl -f $IRCFILE > $WEBDIR/ircreport.txt
$BINDIR/monbackup.pl $WEBDIR/ircreport.txt $LOGDIR/irc
# LOG rawirc
$BINDIR/monbackup.pl $IRCFILE $LOGDIR/rawirc

# xxx - we need to make a sync daemon
# make up for elapsed time spent in omupdate.pl and possibly tcpworm.pl
case $i in
1)
	ETIME=`expr $ETIME + $TTIME + $SYNTIME`
 	STIME=`expr 30 - $ETIME`
	sleep $STIME
	;;
2)
	;;
esac

# end for
done
